/*
		      ANSI C Runtime Library

    Copyright (c) 1983-1997 Green Hills Software, Inc.

    This program is the property of Green Hills Software, Inc,
    its contents are proprietary information and no part of it
    is to be disclosed to anyone except employees of Green Hills
    Software, Inc., or as agreed in writing signed by the President
    of Green Hills Software, Inc.
*/

#if !defined(_MATH_H) && !defined(__NoFloat)
#define _MATH_H

#ifdef __cplusplus
extern "C" {
#endif

#define EDOM    33	/* also #define'd in errno.h */
#define ERANGE  34	/* also #define'd in errno.h */

#define HUGE_VAL       (1.7976931348623158e+308)  /* Max IEEE double exactly */
#ifndef _POSIX_SOURCE
#  define HUGE		HUGE_VAL
#  define HUGEFLOAT	(3.4028235e+38)	/* Max IEEE single exactly */
#endif

#if defined(__STDC__) || defined(__PROTOTYPES__)
# define __ARGS(x)	x
#else
# define __ARGS(x)	()
  double cabs();
#endif

double atof __ARGS((const char *));

double acos	__ARGS((double));
double asin	__ARGS((double));
double atan	__ARGS((double));
double atan2	__ARGS((double, double));
double cos	__ARGS((double));
double sin	__ARGS((double));
double tan	__ARGS((double));

double cosh	__ARGS((double));
double sinh	__ARGS((double));
double tanh	__ARGS((double));

double exp	__ARGS((double));
double frexp	__ARGS((double, int *));
double ldexp	__ARGS((double, int));
double log	__ARGS((double));
double log10	__ARGS((double));
double modf	__ARGS((double, double *));

double pow	__ARGS((double, double));
double sqrt	__ARGS((double));

double ceil	__ARGS((double));
double fabs	__ARGS((double));
double floor	__ARGS((double));
double fmod	__ARGS((double, double));

#if !defined(_WIN32)
/* Microsoft libraries do not support these extensions */
#if defined(__cplusplus) || (__STDC__==0) && !defined(_POSIX_SOURCE)
/* These are all MATH EXTENSIONS */
double acosh	__ARGS((double));	/* neither Ansi nor Unix has these */
double asinh	__ARGS((double));	/* neither Ansi nor Unix has these */
double atanh	__ARGS((double));	/* neither Ansi nor Unix has these */
double erf	__ARGS((double));
double erfc	__ARGS((double));
double j0	__ARGS((double));
double j1	__ARGS((double));
double jn	__ARGS((int, double));
double y0	__ARGS((double));
double y1	__ARGS((double));
double yn	__ARGS((int, double));
double gamma	__ARGS((double));
double hypot	__ARGS((double, double));
int isnan	__ARGS((double));
int isinf	__ARGS((double));
#if defined(__Chorus)
/* The Chorus C library offers these other calls */
double cbrt	__ARGS((double));
double copysign	__ARGS((double, double));
double drem	__ARGS((double));
double expm1	__ARGS((double));
double finite	__ARGS((double));
double lgamma	__ARGS((double));
double log1p	__ARGS((double));
double logb	__ARGS((double));
double rint	__ARGS((double));
double scalb	__ARGS((double));
#endif
#endif /* defined(__cplusplus) || (__STDC__==0) && !defined(_POSIX_SOURCE) */

/* Ansi said these might be added to the standard later, but we have them now */
#ifndef _POSIX_SOURCE
float acosf	__ARGS((float));
float asinf	__ARGS((float));
float atanf	__ARGS((float));
float atan2f	__ARGS((float, float));
float cosf	__ARGS((float));
float sinf	__ARGS((float));
float tanf	__ARGS((float));
float coshf	__ARGS((float));
float sinhf	__ARGS((float));
float tanhf	__ARGS((float));
float expf	__ARGS((float));
float frexpf	__ARGS((float, int *));
float ldexpf	__ARGS((float, int));
float logf	__ARGS((float));
float log10f	__ARGS((float));
float sqrtf	__ARGS((float));
float powf	__ARGS((float, float));
#endif 		/* _POSIX_SOURCE */
#endif /* !defined(_WIN32) */

#if __STDC__==0 && !defined(_POSIX_SOURCE)
#define M_PI	   3.14159265358979323846
#define M_PI_2	   1.570796326794896619231
#define M_PI_4	   0.785398163397448309616
#define M_1_PI	   0.318309886183790671538
#define M_2_PI	   0.636619772367581343076
#define M_E	   2.718281828459045235360
#define M_LN2	   0.69314718055994530942
#define M_LN10	   2.30258509299404568402
#define M_SQRT2	   1.41421356237309504880
#define M_LOG10E   0.434294481903251827651
#define M_LOG2E	   1.442695040888963407
#define M_2_SQRTPI 1.12837916709551257390

#define DOMAIN		1
#define	SING		2
#define	OVERFLOW	3
#define	UNDERFLOW	4
#define	TLOSS		5
#define	PLOSS		6

#ifndef __cplusplus	/* collides with class exception */
    struct exception {
	int type;
	char *name;
	double arg1;
	double arg2;
	double retval;
    };
#endif
#ifndef __REXCEPTION__S
#define __REXCEPTION__S
    struct rexception {
	int type;
	char *name;
	float arg1;
	float arg2;
	float retval;
    };
#endif  /* __REXCEPTION__S */
#endif	/* __STDC__ == 0 and !_POSIX_SOURCE */

#ifdef __cplusplus
	}
// float functions just call double equivalents
inline float ceilf(float _X)
	{return ((float)ceil((double)_X)); }
inline float fabsf(float _X)
	{return ((float)fabs((double)_X)); }
inline float floorf(float _X)
	{return ((float)floor((double)_X)); }
inline float fmodf(float _X, float _Y)
	{return ((float)fmod((double)_X, (double)_Y)); }
inline float modff(float _X, float *_Y)
	{ double _Di, _Df = modf((double)_X, &_Di);
	*_Y = (float)_Di;
	return ((float)_Df); }

// long double functions just call double equivalents
inline long double acosl(long double _X)
	{return (acos((double)_X)); }
inline long double asinl(long double _X)
	{return (asin((double)_X)); }
inline long double atanl(long double _X)
	{return (atan((double)_X)); }
inline long double atan2l(long double _X, long double _Y)
	{return (atan2((double)_X, (double)_Y)); }
inline long double ceill(long double _X)
	{return (ceil((double)_X)); }
inline long double cosl(long double _X)
	{return (cos((double)_X)); }
inline long double coshl(long double _X)
	{return (cosh((double)_X)); }
inline long double expl(long double _X)
	{return (exp((double)_X)); }
inline long double fabsl(long double _X)
	{return (fabs((double)_X)); }
inline long double floorl(long double _X)
	{return (floor((double)_X)); }
inline long double fmodl(long double _X, long double _Y)
	{return (fmod((double)_X, (double)_Y)); }
inline long double frexpl(long double _X, int *_Y)
	{return (frexp((double)_X, _Y)); }
inline long double ldexpl(long double _X, int _Y)
	{return (ldexp((double)_X, _Y)); }
inline long double logl(long double _X)
	{return (log((double)_X)); }
inline long double log10l(long double _X)
	{return (log10((double)_X)); }
inline long double modfl(long double _X, long double *_Y)
	{double _Di, _Df = modf((double)_X, &_Di);
	*_Y = (long double)_Di;
	return (_Df); }
inline long double powl(long double _X, long double _Y)
	{return (pow((double)_X, (double)_Y)); }
inline long double sinl(long double _X)
	{return (sin((double)_X)); }
inline long double sinhl(long double _X)
	{return (sinh((double)_X)); }
inline long double sqrtl(long double _X)
	{return (sqrt((double)_X)); }
inline long double tanl(long double _X)
	{return (tan((double)_X)); }
inline long double tanhl(long double _X)
	{return (tanh((double)_X)); }

// overloads
inline double abs(double _X)
	{return (fabs(_X)); }
inline double pow(double _X, int _Y)
	{typedef double _Ty;
	unsigned int _N = _Y;
	if (_Y < 0)
		_N = -_N;
	for (_Ty _Z = _Ty(1); ; _X *= _X)
		{if ((_N & 1) != 0)
			_Z *= _X;
		if ((_N >>= 1) == 0)
			return (_Y < 0 ? _Ty(1) / _Z : _Z); }}
inline int pow(int _X, int _Y)
	{typedef int _Ty;
	unsigned int _N = _Y;
	if (_Y < 0)
		_N = -_N;
	for (_Ty _Z = _Ty(1); ; _X *= _X)
		{if ((_N & 1) != 0)
			_Z *= _X;
		if ((_N >>= 1) == 0)
			return (_Y < 0 ? _Ty(1) / _Z : _Z); }}
inline float abs(float _X)
	{return (fabsf(_X)); }
#if defined(_WIN32)
/* Microsoft libraries do not have float versions of these functions. */
inline float acosf(float _X)	{return ((float)acos((double)_X)); }
inline float asinf(float _X)	{return ((float)asin((double)_X)); }
inline float atanf(float _X)	{return ((float)atan((double)_X)); }
inline float atan2f(float _X, float _Y)	{return ((float)atan2((double)_X, (double)_Y)); }
inline float cosf(float _X)	{return ((float)cos((double)_X)); }
inline float sinf(float _X)	{return ((float)sin((double)_X)); }
inline float tanf(float _X)	{return ((float)tan((double)_X)); }
inline float coshf(float _X)	{return ((float)cosh((double)_X)); }
inline float sinhf(float _X)	{return ((float)sinh((double)_X)); }
inline float tanhf(float _X)	{return ((float)tanh((double)_X)); }
inline float expf(float _X)	{return ((float)exp((double)_X)); }
inline float frexpf(float _X, int *_Y)	{return ((float)frexp((double)_X, _Y)); }
inline float ldexpf(float _X, int _Y)	{return ((float)ldexp((double)_X, _Y)); }
inline float logf(float _X)	{return ((float)log((double)_X)); }
inline float log10f(float _X)	{return ((float)log10((double)_X)); }
inline float sqrtf(float _X)	{return ((float)sqrt((double)_X)); }
inline float powf(float _X, float _Y)	{return ((float)pow((double)_X, (double)_Y)); }
#endif
inline float acos(float _X)
	{return (acosf(_X)); }
inline float asin(float _X)
	{return (asinf(_X)); }
inline float atan(float _X)
	{return (atanf(_X)); }
inline float atan2(float _Y, float _X)
	{return (atan2f(_Y, _X)); }
inline float ceil(float _X)
	{return (ceilf(_X)); }
inline float cos(float _X)
	{return (cosf(_X)); }
inline float cosh(float _X)
	{return (coshf(_X)); }
inline float exp(float _X)
	{return (expf(_X)); }
inline float fabs(float _X)
	{return (fabsf(_X)); }
inline float floor(float _X)
	{return (floorf(_X)); }
inline float fmod(float _X, float _Y)
	{return (fmodf(_X, _Y)); }
inline float frexp(float _X, int * _Y)
	{return (frexpf(_X, _Y)); }
inline float ldexp(float _X, int _Y)
	{return (ldexpf(_X, _Y)); }
inline float log(float _X)
	{return (logf(_X)); }
inline float log10(float _X)
	{return (log10f(_X)); }
inline float modf(float _X, float * _Y)
	{return (modff(_X, _Y)); }
inline float pow(float _X, float _Y)
	{return (powf(_X, _Y)); }
inline float pow(float _X, int _Y)
	{typedef float _Ty;
	unsigned int _N = _Y;
	if (_Y < 0)
		_N = -_N;
	for (_Ty _Z = _Ty(1); ; _X *= _X)
		{if ((_N & 1) != 0)
			_Z *= _X;
		if ((_N >>= 1) == 0)
			return (_Y < 0 ? _Ty(1) / _Z : _Z); }}
inline float sin(float _X)
	{return (sinf(_X)); }
inline float sinh(float _X)
	{return (sinhf(_X)); }
inline float sqrt(float _X)
	{return (sqrtf(_X)); }
inline float tan(float _X)
	{return (tanf(_X)); }
inline float tanh(float _X)
	{return (tanhf(_X)); }
inline long double abs(long double _X)
	{return (fabsl(_X)); }
inline long double acos(long double _X)
	{return (acosl(_X)); }
inline long double asin(long double _X)
	{return (asinl(_X)); }
inline long double atan(long double _X)
	{return (atanl(_X)); }
inline long double atan2(long double _Y, long double _X)
	{return (atan2l(_Y, _X)); }
inline long double ceil(long double _X)
	{return (ceill(_X)); }
inline long double cos(long double _X)
	{return (cosl(_X)); }
inline long double cosh(long double _X)
	{return (coshl(_X)); }
inline long double exp(long double _X)
	{return (expl(_X)); }
inline long double fabs(long double _X)
	{return (fabsl(_X)); }
inline long double floor(long double _X)
	{return (floorl(_X)); }
inline long double fmod(long double _X, long double _Y)
	{return (fmodl(_X, _Y)); }
inline long double frexp(long double _X, int * _Y)
	{return (frexpl(_X, _Y)); }
inline long double ldexp(long double _X, int _Y)
	{return (ldexpl(_X, _Y)); }
inline long double log(long double _X)
	{return (logl(_X)); }
inline long double log10(long double _X)
	{return (log10l(_X)); }
inline long double modf(long double _X, long double * _Y)
	{return (modfl(_X, _Y)); }
inline long double pow(long double _X, long double _Y)
	{return (powl(_X, _Y)); }
inline long double pow(long double _X, int _Y)
	{typedef long double _Ty;
	unsigned int _N = _Y;
	if (_Y < 0)
		_N = -_N;
	for (_Ty _Z = _Ty(1); ; _X *= _X)
		{if ((_N & 1) != 0)
			_Z *= _X;
		if ((_N >>= 1) == 0)
			return (_Y < 0 ? _Ty(1) / _Z : _Z); }}
inline long double sin(long double _X)
	{return (sinl(_X)); }
inline long double sinh(long double _X)
	{return (sinhl(_X)); }
inline long double sqrt(long double _X)
	{return (sqrtl(_X)); }
inline long double tan(long double _X)
	{return (tanl(_X)); }
inline long double tanh(long double _X)
	{return (tanhl(_X)); }
#endif /* __cplusplus */
#undef __ARGS
#endif /* !defined(_MATH_H) && !defined(__NoFloat) */
