/*
		      ANSI C Runtime Library

    Copyright (c) 1983-1997 Green Hills Software, Inc.

    This program is the property of Green Hills Software, Inc,
    its contents are proprietary information and no part of it
    is to be disclosed to anyone except employees of Green Hills
    Software, Inc., or as agreed in writing signed by the President
    of Green Hills Software, Inc.
*/

#ifndef _STDIO_H
#define _STDIO_H
#ifdef __cplusplus
extern "C" {
#endif

#if defined(__Chorus)
#include <c_types.h>
#else
#if defined(__cplusplus) && !defined(NULL)
#define NULL 0L		/* C++ requires this to be integral, not a pointer */
#elif !defined(NULL)
#define NULL ((void *)0)/* locale.h stddef.h stdio.h stdlib.h string.h time.h */
#endif

#ifndef _SIZE_T
#define _SIZE_T
# if defined(__Ptr_Is_64) && !defined(__Int_Is_64)
typedef unsigned long size_t;
# else
typedef unsigned int size_t;
# endif
#endif	/* _SIZE_T */
#endif	/* defined(__Chorus) */

#if defined(__STDC__) || defined(__cplusplus) /* || defined(__PROTOTYPES__) */
#include <stdarg.h>
#endif

typedef struct iobuf {
    unsigned char *io_next;	/* next character to be read from buffer or
				   next position to put a character in buffer */
    unsigned char *io_base;	/* start of buffer */
    long  io_left;		/* number of characters left in buffer */
    unsigned io_channel:14;	/* OS I/O channel number */ /* GWW: was short */
    unsigned io_ivbuf: 1;	/* the read buffer differs from disk (ungetc called) */
    unsigned io_append: 1;	/* should do seeks to eof before writing */
    char  io_tmp;		/* used by ungetc on unbuffered files */
    unsigned io_buffering:2;	/* _IONBF,_IOFBF,_IOLBF */
    unsigned io_eof: 1; 	/* have read end of file */
    unsigned io_error:1;	/* have detected io error in file */
    unsigned io_stdio_buffer:1; /* buffer for this file created by stdio */
    unsigned io_readable:1;	/* file may be read at this time */
    unsigned io_writable:1;	/* file may be written at this time */
    unsigned io_readwrite:1;	/* file opened for both reading and writing */
} FILE;

#ifndef _FPOS_T
#define _FPOS_T
typedef int fpos_t;
#endif

#define _IOFBF		0
#define _IOLBF		1
#define _IONBF		2
#define _IOSBF		3

#define EOF		(-1)
#if !defined(__unix) || defined(__gmicro)
# define BUFSIZ		512
# define FOPEN_MAX	20
# define L_tmpnam	16
#else
# define BUFSIZ		1024
# define FOPEN_MAX	40
# define L_tmpnam	25
# if __STDC__==0 && !defined(POSIX_SOURCE)
#  define P_tmpdir	"/usr/tmp/"
# endif
#endif	/* !unix */
#define FILENAME_MAX	1023

#define SEEK_CUR	1
#define SEEK_END	2
#define SEEK_SET	0

#ifdef __unix
#define TMP_MAX 	17576		/* 26 * 26 * 26 */
#else
#define TMP_MAX 	1048576		/* 32 * 32 * 32 * 32 */
#endif

#define stdin (&_iob[0])
#define stdout (&_iob[1])
#define stderr (&_iob[2])

/* variables */
#pragma ghs startdata
extern FILE _iob[ /* FOPEN_MAX */ ];
#ifdef __ghs_thread_safe
extern void *_iob_lock[ FOPEN_MAX ]; /* per-IOB locking */
#endif
#pragma ghs enddata

/* functions */
#if defined(__STDC__) || defined(__PROTOTYPES__)
#  define __ARGS(x)		x
#else
#  define __ARGS(x)		()
#  define BLOCKSIZE	2048	/* block size to allocate with sbrk in malloc */
   extern long strtol(), getl(), putl();
   extern int eprintf();
#endif

int remove __ARGS((const char *));
int rename __ARGS((const char *, const char *));
FILE *tmpfile __ARGS((void));
char *tmpnam __ARGS((char *));

int _flsbuf __ARGS((int, FILE *));
int _filbuf __ARGS((FILE *));
int fclose __ARGS((FILE *));
int fflush __ARGS((FILE *));
FILE *fopen __ARGS((const char *, const char *));
FILE *freopen __ARGS((const char *, const char *, FILE *));
FILE *fdopen __ARGS((int, const char *));
void setbuf __ARGS((FILE *, char *));
int setvbuf __ARGS((FILE *, char *, int, size_t ));
int fprintf __ARGS((FILE *, const char *, ...));
int fscanf __ARGS((FILE *, const char *, ...));
int printf __ARGS((const char *, ...));
int scanf __ARGS((const char *, ...));
int sprintf __ARGS((char *, const char *, ...));
int sscanf __ARGS((const char *, const char *, ...));

int fgetc __ARGS((FILE *));
char *fgets __ARGS((char *, int, FILE *));
int fputc __ARGS((int, FILE *));
int fputs __ARGS((const char *, FILE *));
int getc __ARGS((FILE *));
int getchar __ARGS((void));
char *gets __ARGS((char *));
int putc __ARGS((int, FILE *));
int putchar __ARGS((int ));
int puts __ARGS((const char *));
int ungetc __ARGS((int, FILE *));
size_t fread __ARGS((void *, size_t, size_t, FILE *));
size_t fwrite __ARGS((const void *, size_t, size_t, FILE *));

int fgetpos __ARGS((FILE *, fpos_t *));
int fseek __ARGS((FILE *, long int, int ));
int fsetpos __ARGS((FILE *, const fpos_t *));
long int ftell __ARGS((FILE *));
void rewind __ARGS((FILE *));

void clearerr __ARGS((FILE *));
int feof __ARGS((FILE *));
int ferror __ARGS((FILE *));
void perror __ARGS((const char *));

/* don't prototype these functions in this mode rather than #include varargs */
#if !defined(__STDC__) && !defined(__cplusplus) && defined(__PROTOTYPES__)
#undef  __ARGS
#define __ARGS(x)	()
#endif
int vfprintf __ARGS((FILE *, const char *, va_list ));
int vprintf __ARGS((const char *, va_list ));
int vsprintf __ARGS((char *, const char *, va_list ));
int vfscanf __ARGS((FILE *, const char *, va_list ));
int vscanf __ARGS((const char *, va_list ));
int vsscanf __ARGS((const char *, const char *, va_list ));

#define getchar()	getc(stdin)
#define putchar(ch)	putc((ch),stdout)
#define fileno(file)	((file)->io_channel)
#define _fileno(file)	((file)->io_channel)	 /* this is a valid ANSI name */
#define feof(file)	((file)->io_eof)
#define ferror(file)	((file)->io_error)

/* DAVEK Tue Jul 21 19:24:08 PDT 1998 getc/putc macros not threadsafe. 
 * Use functions instead */
#ifndef __ghs_thread_safe

  /* non-threadsafe macros */

# define putc(c,f)	(--(f)->io_left<0?_flsbuf((c),f):(*(f)->io_next++=(c)))
# define getc(f) 	(--(f)->io_left<0?_filbuf(f):*(f)->io_next++)
  /* unlocked versions included for completeness 
   * and for simpler library implementation */
# define putc_unlocked(c,f) 	putc(c,f)
# define getc_unlocked(f) 	getc(f)
# define getchar_unlocked()	getchar()
# define putchar_unlocked(ch)	putchar(ch)
# define clearerr(file)	(void) ((file)->io_eof=0,(file)->io_error=0)

#else		/* __ghs_thread_safe */

# define putc_unlocked(c,f) 	(--(f)->io_left<0?_flsbuf((c),f):\
					(*(f)->io_next++=(c)))
# define getc_unlocked(f) 	(--(f)->io_left<0?_filbuf(f):*(f)->io_next++)
# define getchar_unlocked()	getc_unlocked(stdin)
# define putchar_unlocked(ch)	putc_unlocked((ch),stdout)

/* POSIX-style locking functions used for threadsafe stdio */
void	flockfile __ARGS((FILE *));
int	ftrylockfile __ARGS((FILE *));
void	funlockfile __ARGS((FILE *));
void    flockcreate __ARGS((FILE *));
void    flockdestroy __ARGS((FILE *));

#endif


#if defined(_POSIX_SOURCE)
#  define L_ctermid	9
#  define L_cuserid	9
#elif __STDC__ == 0 || defined(c_plusplus)
char *tempnam	__ARGS((const char *, const char *));
FILE *popen	__ARGS((const char *, const char *));
int pclose	__ARGS((FILE*));
int getw	__ARGS((FILE*));
#endif
#undef __ARGS
#ifdef __cplusplus
}
#endif

#endif	/* _STDIO_H */
