/*
		      ANSI C Runtime Library

	Copyright (c) 1983-1997 Green Hills Software, Inc.

    This program is the property of Green Hills Software, Inc,
    its contents are proprietary information and no part of it
    is to be disclosed to anyone except employees of Green Hills
    Software, Inc., or as agreed in writing signed by the President
    of Green Hills Software, Inc.
*/

#ifndef _STDLIB_H
#define _STDLIB_H

#ifdef __cplusplus
extern "C" {
#endif

#if defined(__Chorus)
#include <c_types.h>
#else
#if defined(__cplusplus) && !defined(NULL)
#define NULL 0L		/* C++ requires this to be integral, not a pointer */
#elif !defined(NULL)
#define NULL ((void *)0)/* locale.h stddef.h stdio.h stdlib.h string.h time.h */
#endif

#ifndef _SIZE_T
#define _SIZE_T
# if defined(__Ptr_Is_64) && !defined(__Int_Is_64)
typedef unsigned long size_t;
# else
typedef unsigned int size_t;
# endif
#endif	/* _SIZE_T */
#endif	/* defined(__Chorus) */

#ifndef _WCHAR_T
#if defined(__ghs) && defined(__Chorus)
#  include <runetype_f.h>
#else /* defined(__ghs) && defined(__Chorus) */
	typedef 
# ifdef __WChar_Is_Unsigned__
		unsigned 
# endif
# ifdef __WChar_Is_Short__
			short
# elif __WChar_Is_Int__
			int
# else
			long 
# endif
				wchar_t;
#endif /* defined(__Chorus) */
#define _WCHAR_T
#endif /* _WCHAR_T */

#ifndef _DIV_T
#define _DIV_T
typedef struct {
    int quot;		/* quotient */
    int rem;		/* remainder */
} div_t;
#endif
#ifndef _LDIV_T
#define _LDIV_T
typedef struct {
    long quot;		/* quotient */
    long rem;		/* remainder */
} ldiv_t;
#endif

#define EXIT_FAILURE	    1
#define EXIT_SUCCESS	    0
/* #define RAND_MAX	    (2029052025-1) */
#define RAND_MAX	    (0x7fff)
#define MB_CUR_MAX	    sizeof(wchar_t)

#if __STDC__ == 0
#define BLOCKSIZE	2048	/* block size to allocate with sbrk in malloc */
#endif

#if defined(__STDC__) || defined(__cplusplus)
#define __ARGS(x)		x
#define __VOIDPTR		void
#elif defined(__PROTOTYPES__)
#define __ARGS(x)		x
#define __VOIDPTR		char
#else
#define __ARGS(x)		()
#define __VOIDPTR		char
#endif

#if defined(__cplusplus)
inline int  abs(int j) { return j<0 ? -j: j; }
inline long labs(long j) { return j<0 ? -j: j; }
#else
   int abs __ARGS((int));
   long int labs __ARGS((long int));
#  define abs(j)	((j)<0?-(j):(j))
#  define labs(j)	((j)<0?-(j):(j))
/* the compiler recognizes the above construct and will produce better code */
#endif

void		abort __ARGS((void));
int		atexit __ARGS((void (*)(void)));
#ifndef __NoFloat
double		atof __ARGS((const char *));
#endif
int		atoi __ARGS((const char *));
long		atol __ARGS((const char *));
__VOIDPTR*	bsearch __ARGS((const void *, const void *, size_t, size_t,
				int  (*)(const void *, const void *)));
__VOIDPTR*	calloc __ARGS((size_t, size_t));

div_t		div __ARGS((int, int));

void		exit __ARGS((int));
void		free __ARGS((void *));
char*		getenv __ARGS((const char *));
ldiv_t		ldiv __ARGS((long, long));
__VOIDPTR*	malloc __ARGS((size_t));
int		mblen __ARGS((const char *, size_t));
size_t		mbstowcs __ARGS(( wchar_t *, const char *, size_t));
int		mbtowc __ARGS((wchar_t *, const char *, size_t));
void		qsort __ARGS((void *, size_t, size_t,
			      int (*)(const void*, const void*)));
int		rand   __ARGS((void));
int		rand_r __ARGS((long *));
__VOIDPTR*	realloc __ARGS((void *, size_t));
void		srand __ARGS((unsigned int));
#ifndef __NoFloat
double		strtod __ARGS((const char *, char **));
#endif
long		strtol __ARGS((const char *, char **, int));
unsigned long	strtoul __ARGS((const char *, char **, int));
int		system __ARGS((const char *));
size_t		wcstombs __ARGS(( char *, const wchar_t *, size_t));
int		wctomb __ARGS((char *, wchar_t));
#if __STDC__ == 0
void		swab __ARGS((const char *, char *, int));
#endif
#undef __ARGS
#undef __VOIDPTR

#ifdef __cplusplus
}

extern "C++" {
inline long abs(long _X) 	    {return (labs(_X)); }
inline ldiv_t div(long _X, long _Y) {return (ldiv(_X, _Y)); }
}
#ifndef __NoFloat
inline long double strtold(const char *_S, char **_P) {return (strtod(_S, _P));}
#endif /* __NoFloat */
#endif /* __cplusplus */

#endif			   /* _STDLIB_H */
