/*
   Manipulate the LEDs on the M.Core MMC2001
   High-Performance Processor Evaluation Board (MMCCMB1200).

   Examine the readme.txt file in this directory for more
   information.

   REVISION HISTORY
     
   12 Aug 1998 Initial version of this file
     
   Copyright 1998 Green Hills Software, Inc.
 
*/

#include <stdio.h>

struct led_struct {
    volatile unsigned short led;
             unsigned short u0;
             unsigned short u1;
             unsigned short u2;
};

struct led_struct *leds = (struct led_struct *)0x10005000;

int transition_leds(int);	/* prototype */

int main(void)
{
    int i;
    char *msg;

    for(i=0;i<4;++i) {
	leds[i].led = 0x0040; /* Turn on  ST[i] */
	leds[i].led = 0x00C0; /* Turn off ST[i] */
    }
    msg = "M.Core MMC2001 Processor";

    (void)puts(msg);
    (void)puts("M.Core CMB1200 Evaluation board\n"
	       "Programs running on the \n"
	       " target can use stdio \n"
	       " library calls to perform \n"
	       " I/O on the HOST computer! \n");


    for(i=100; i; --i) {
	transition_leds(1);
    }

    (void)puts(msg);
    (void)puts("Blinking LED demonstration");
    (void)puts("Complete.");
    
    return 0;
}

#define DISPLAY_LOOP_MAX 100	/* Determines what brightest setting is. */

#define LOW 0
#define  HI 100

int settings[][4] = {
 { HI, LOW, LOW, LOW},
 {LOW,  HI, LOW,  HI},
 {LOW, LOW,  HI, LOW}
};
#undef LOW
#undef HI

int current_settings[] =
 {  0,   0,   0,   0};

int previous_setting_index;
int desired_setting_index = 0;

#define TRANSITION_STEPS 128

void delay (int i) {
    volatile int j=i;
    while(--j)
	;
}


/* The LED's brightness doesn't seem to increase linearly with
   time spent on.  One option would be to make the function
   nonlinear with it first going linearly from (0,0) to (A, 100-A)
   and then linearly from there to (100,100).

   Another simpler option is to make it parabolic,
   where y=x*x/100.
 */

int transition_leds(int transition_delay) {
    int i, j, transition, display;

    previous_setting_index = desired_setting_index;
    ++desired_setting_index;
    if(desired_setting_index == sizeof(settings)/sizeof(settings[0]))
	desired_setting_index = 0;

    for(transition = 0; transition < TRANSITION_STEPS; ++transition) {
	for(j=0;j<4;++j) {
	    current_settings[j] = settings[previous_setting_index][j];
	    current_settings[j] += transition *
		(settings[desired_setting_index][j] -
		settings[previous_setting_index][j]) / TRANSITION_STEPS;
	    /* Make transition nonlinear */
	    current_settings[j] *= current_settings[j];
	    current_settings[j] /= DISPLAY_LOOP_MAX;
	}

	for(display = DISPLAY_LOOP_MAX; display; --display) {
	    for(j=0;j<4;++j) {
		if(current_settings[j] >= display)
		    leds[j].led = 0x0040; /* Turn on  ST[j] */
		else
		    leds[j].led = 0x00C0; /* Turn off ST[j] */
	    }
	    delay(transition_delay);
	}
    }
    return 0;
}

