/**********************************************************************
file:		sortbyname.c		Copyright APh 1992,93,94,95
summary:	short program to demonstrate basic features of MULTI
description:
	read lines with names and ages from a file into a linked list
	sort the list alphabetically by name
	print the list
	
contents:
	sortbyname(argc,filename)	read name/age pairs, sort by name and print
	xcmp(p,q)			compare two structs for xsort
**********************************************************************/
#include <stdio.h>
#define MAXN 1024*1024
enum sex {undetermined, male, female};

/* externals and forward references */
extern char *malloc();
extern FILE *fopen();
char *fgets();

/* globals */
int MAIN_=0;
int xcmp();

/* structure and structure ptrs for linked list */
struct element {
	char *name;
	unsigned int age:4;
	enum sex sex:2;
	struct element *next;
};

/* program to read and alphabetize name/age pairs */
sortbyname(int argc, char *filename)
{
	FILE *fp;
	int age,cnt,slen;
	char name[256],linebuf[256], buf[BUFSIZ];
	struct element *pfirst,*pprev,*pthis, *pnext;
	struct element **plist,**pp;	
	_main();
	
	/* open file containing data records (format: "name age\n") */
	if(argc>1) fp=fopen(filename,"r");
	else fp=fopen("roster","r");
	if(fp==0) {
		printf("couldn't open file %s\n",filename);
		exit(1);
	}
	setbuf(fp, buf);
		
	/* read records into a linked list of structs */
	for(pfirst=0, cnt=0; cnt<MAXN; cnt++) {
		fgets(linebuf,sizeof(linebuf),fp);
		if(sscanf(linebuf,"%s %d",name,&age)!=2) break;
		pthis = (struct element *) malloc(sizeof(struct element));
		slen=strlen(name);
		pthis->name=malloc(slen);
		strcpy(pthis->name,name);
		pthis->age=age;
		pthis->next=0;
		if(pfirst==0)
			pfirst=pthis;
		else
			pprev->next=pthis;
		pprev=pthis;
	}
	
	/* make an array of ptrs to structs */
	pp=plist=(struct element **) malloc(cnt*sizeof(struct element *));
	for(*pp++=pthis=pfirst; pthis->next; pthis=pthis->next)
		*pp++=pthis->next;

	/* sort the structs */
	xsort_(plist,&cnt,xcmp);
	
	/* print sorted list of names */
	printnames(plist, cnt);
	
	/* release allocated memory */
	for(pthis=pfirst; pthis!=0; pthis=pnext) {
		pnext = pthis->next;
		free(pthis);		/* free the linked list */
	}	
	free(plist);			/* free table of ptrs */
	free(pp);			/* free table of sorted ptrs (oops)*/
}

/**********************************************************************
function:	xcmp(p,q)
summary:	compare to structs for xsort_()
description:
	return	1 if first arg greater than second
			0 otherwise	
**********************************************************************/
xcmp(struct element **p, struct element **q)
{
	return(strcmp((*p)->name,(*q)->name));
}

#if 0
/**********************************************************************
function:	printnames(plist,cnt)
summary:	print list of name age pairs the hard way.
		suitable when demo'ing native UNIX target.
**********************************************************************/
printnames(struct element **plist, int cnt)
{
	int pid, fd[2];
	struct element **pp;
	char sbuf[256];
	
	/* fork a process to process sorted list */
	pipe(fd);
	if((pid=fork())) {

		/* parent process sends records to child */
		for(pp=plist; pp-plist<cnt; pp++) {
			sprintf(sbuf,"%s %d\n\0",(*pp)->name, (*pp)->age);
			write(fd[1],sbuf,strlen(sbuf));
		}
		close(fd[0]);
		close(fd[1]);

	} else {

		/* child process invokes program to process sorted records */
		execl("mprint","mprint",(char *)0);
	}
}
#endif
