#include <stdio.h>

/* The "_rnerr" funcion will be called whenever a run-time error has occurred
 * and runtime error checking is enabled 
 */
void _rnerr(int code, int line, const char *filename)
{
/*
 * code:       error type  
 * line:       line in the source code which fails  
 * filename:   name of the source file which fails  
 *          
 * Valid error types:  
 * 0:  UNUSED  
 * 1:  array index  
 * 2:  variant  
 * 3:  nil pointer  
 * 4:  switch out of bounds  
 * 5:  uninitialized variable  
 * 6:  assignment out of bounds  
 * 7:  division by zero  
 * 8:  value out of bounds (pred,succ)  
 * 9:  free of something not malloced  
 * 10: free of something already free  
 * 11: free list corrupted  
 * 12: reference to bad data  
 * 13: unreferenced memory  
 * 14: overflow  
 * 15: Error limit exceeded  
 * 16: Watchpoint  
 * 17: Return of nothing from non-void procedure  
 * 18: Ambiguous virtual function called  
 * 19: Attempt to free NULL  
 * 20: Heap max exceeded  
 * 21: Stack max exceeded    
 */

 #if 1
    switch(code)  {
    case 1:
	printf("Runtime error: Array index out of bounds in file = %s, line = %d\n", filename, line);
	break;
	
    case 3:
	printf("Runtime error: NULL pointer dereference in file = %s, line = %d\n", filename, line);
	break;
	
    case 4:
	printf("Runtime error: Case/Switch has no match in file = %s, line = %d\n", filename, line);
	break;
	
    case 7:
	printf("Runtime error: Divide by zero!!! in file = %s, line = %d\n", filename, line);
	break;
	
    case 9:
	printf("Runtime error: Attempt to free unallocated memory in file = %s, line = %d\n", filename, line);
	break;
	
    case 19:
	printf("Runtime error: Attempt to free memory not allocated in file = %s, line = %d\n", filename, line);
	break;
	
    default:        
	printf("Error code = %d, file = %s, line = %d\n", code, filename, line);
    }
#endif
    return;
}
