/*
 *  C Runtime library Checksum Verification
 *
 *  Copyright 1983,1984,1985,1986.1987,1988,1989,1990,1991,1992,1993,1994
 *  by Green Hills Software Inc.
 *
 *  This program is the property of Green Hills Software, Inc,
 *  its contents are proprietary information and no part of it
 *  is to be disclosed to anyone except employees of Green Hills
 *  Software, Inc., or as agreed in writing signed by the President
 *  of Green Hills Software, Inc.
 */

/* Call __cksumsects immediately after download (before changing any
 * data values) to verify the checksum against the values inserted by the
 * linker.
 * __cksumsects returns 0 on success, -1 on error.
 */

#include <stdio.h>
extern struct psinfo {
    union {
	int nonzero;
	char sectname[8];
    } f;
    long addr;
    long size;
    int flags;
#define F_TEXT 0x20
#define F_DATA 0x40
#define F_BSS  0x80
} __psinfo[];


/* Wed Nov 23 11:15:01 PST 1994 - Algorithm courtesy L.Diehr of Ford,
based on Joe Campbell (1987); "C Programmer's Guide to Serial Communications",
figure 19-5.
*/
#define CRC_POLYNOMIAL 0x10211021
static int
cksum (s, len)
unsigned char *s;
int len;
{
    int csum=0;
    int data;
    int i;
    while (len--) {
	data = *s++ << 24;		/* Left justify character */
	for (i=0; i<8; ++i) {		/* Loop for each bit in char */
	    if ((data^csum) & 0x80000000) {	/* Shift CRC, feed back hi bit */
		csum = (csum<<1) ^ CRC_POLYNOMIAL;
	    } else {
		csum <<= 1;
	    }
	    data <<= 1;
	}
    }
    return csum;
}

/* Checksum text and data sections.  Return 0 on success, -1 on error. */
int __cksumsects()
{
    struct psinfo *ps;
    unsigned long addr, size;
    int flags;
    int csum1, csum2;

    for (ps=__psinfo; ps->f.nonzero; ++ps) {
	addr = ps->addr;
	size = ps->size;
	flags = ps->flags;
	if (size && (flags & (F_DATA|F_TEXT))) {
	    csum1 = cksum((unsigned char *)addr,size-4);
	    csum2 = *(int *)(addr+size-4);
	    if (csum1 != csum2) 
		return -1;
	}
    }
    return 0;
}
