/*
 *            Debugger
 *
 *    Copyright 1990-1996 Green Hills Software, Inc.
 *
 *    This program is the property of Green Hills Software, Inc,
 *    its contents are proprietary information and no part of it
 *    is to be disclosed to anyone except employees of Green Hills
 *    Software, Inc., or as agreed in writing signed by the President
 *    of Green Hills Software, Inc.
 */
/* ind_bcnt.c: compiler internal functions for Block Coverage Profiling. */

#if defined(EMBEDDED)

#include "indos.h"
#include "ind_io.h"
#include "ind_exit.h"
#include "ind_thrd.h"

struct blk { char *addr; unsigned int cnt; };
struct bhdr { int len; struct blk *info; int inserted; struct bhdr *next; };
static struct bhdr *blocklisthead;

static void dumpinfo(void) 
{
    int fd;
    struct bhdr *test;

    fd = creat("bmon.out", 0644);
    if (fd == -1) {
	static const char creatfailed[] = "bcount: could not create bmon.out\n";
	write(2, creatfailed, sizeof(creatfailed)-1);
	return;
    }
    for (test = blocklisthead; test; test = test->next) {
	static const char writefailed[] = "bcount: write failed for bmon.out\n";
#if defined(alpha) || defined(__alpha)
/* special processing for long pointers on Alpha */
	unsigned int i = 0, u = 0;
	unsigned int us[16*2];		/* must be even number of elements */
	struct blk *b = test->info;
	while (i < test->len) {
	    unsigned int uc;
	    us[u++] = (unsigned int) b->addr;
	    us[u++] = b->cnt;
	    ++b, ++i;
	    uc = sizeof(us[0])*u;
	    if ((i == test->len || (sizeof(us) == uc)) &&
			write(fd, us, uc) != uc) {
		write(2, writefailed, sizeof(writefailed)-1);
		close(fd);
		return;
	    }
	}
#else
	int cnt = sizeof(struct blk)*test->len;
	if (write(fd, test->info, cnt) != cnt) {
	    write(2, writefailed, sizeof(writefailed)-1);
	    close(fd);
	    return;
	}
#endif
    }
    close(fd);
}

void __ghs_bcount(struct bhdr *routinfo, char *routaddr)
{
    __ghsLock();
    if (routinfo->inserted) {
	__ghsUnlock();
	return;
    }
    if (!blocklisthead) {
	static struct __GHS_AT_EXIT gae;
	gae.func = dumpinfo;
	__ghs_at_exit(&gae);
    }

    routinfo->inserted = 1;
    routinfo->next = blocklisthead;
    blocklisthead = routinfo;
    __ghsUnlock();
}
#endif /* defined(EMBEDDED) */
