/*
		    ANSI C Runtime Library
	
	Copyright 1983-1996 Green Hills Software, Inc.

    This program is the property of Green Hills Software, Inc,
    its contents are proprietary information and no part of it
    is to be disclosed to anyone except employees of Green Hills
    Software, Inc., or as agreed in writing signed by the President
    of Green Hills Software, Inc.
*/

/*----------------------------------------------------------------------*/
/* ind_crt0.c: Machine Independent startup code.			*/
/*									*/
/* CALLED FROM:  _start in crt0.o assembly  				*/
/* ENTRY POINT:  __ghs_ind_crt0 					*/
/*									*/
/* WARNING! DO NOT COMPILE THIS FILE WITH PROFILING OR MEMORY CHECKING.	*/
/*									*/
/* This is the first C function called by the standard libraries.	*/
/* First it discovers where sections are located.  Then it initializes	*/
/* static and global data in conformance with what ANSI C requires.	*/
/* Since the profiling and memory checking code depend on initialized	*/
/* global variables, which this routine initializes, DO NOT enable the	*/
/* profiling or memory checking options when compiling this file.	*/
/* Since global register variables are initialized BEFORE this routine	*/
/* is called, you MUST compile this routine with -globalreg=max.	*/
/*									*/
/* This code relocates initialized pointers in PIC, PIR, or PID modes.	*/
/* (PIC, PIR, PID = Position-Independent Code, Read-Only Data, Data.)	*/
/*									*/
/* The first argument to __ghs_ind_crt0 is baseptr, which is an array	*/
/* defined in the assembly startup file, crt0.<CPU>:			*/
/*									*/
/* baseptr[0] = base of text		(= __ghsbegin_picbase)		*/
/* baseptr[1] = base of read-only data	(= __ghsbegin_robase)		*/
/* baseptr[2] = base of read-write data	(= __ghsbegin_pidbase)		*/
/*									*/
/* Currently, these linktime addresses are just weak references.	*/
/* They could also be real global symbols.  The names defined in the	*/
/* crt0.<CPU> assembly language file and passed in the baseptrs array	*/
/* must match the names used to calculate PICBASE, PIRBASE, PIDBASE.	*/
/*									*/
/* The symbol __ghsbegin_picbase below is declared as a text symbol	*/
/* (using a function declaration).  The compiler automatically adds	*/
/* the PIC base register to the reference to this symbol.  We subtract	*/ 
/* the linktime address (in the baseptrs array) to obtain the PICBASE,	*/
/* which will then be added to initialized pointers to text locations.	*/
/* We compute PIRBASE and PIDBASE similarly.				*/
/*									*/
/* When a PIC/PIR/PID pointer relocation is needed, the compiler puts	*/
/* data into two special data sections (.fixtype and .fixaddr) which	*/
/* the code below uses to relocate the initialized pointers by the	*/
/* appropriate offset.  See the detailed comments on PIC/PIR/PID below.	*/
/*									*/
/* If a target does not	support PIC/PIR/PID, the program uses absolute	*/
/* addresses and initialized pointers are not relocated.		*/
/*----------------------------------------------------------------------*/

/*----------------------------------------------------------------------*/
/* NOTES on clearing BSS and ROM to RAM data copying.			*/
/*									*/
/* The BSS section attribute or the CLEAR keyword in the .lnk file mark	*/
/* sections (called BSS sections) to be set to 0 before main is called.	*/
/* The NOCLEAR keyword can be used to override the BSS attribute.	*/
/*									*/
/* The ROM keyword in the .lnk file tells the linker to create a ROM	*/
/* section to be copied from ROM to RAM before main is called.		*/
/*									*/
/* The linker creates a read-only data section called .secinfo to hold	*/
/* information about BSS and ROM sections.  The .secinfo data section	*/
/* is divided into parts.  One part of .secinfo contains pointers to	*/
/* (and sizes of) all BSS sections.  Another part of .secinfo contains	*/
/* pointers to (and sizes of) all ROM sections, plus pointers to the	*/
/* corresponding RAM target sections.  The code below uses the .secinfo	*/
/* information to clear BSS sections and to copy ROM sections to RAM.	*/
/*									*/
/* The pointers in .secinfo are all linktime addresses.  The code below	*/
/* explicitly adds PICBASE to address ROM and PIDBASE to address RAM.	*/
/* The addition of PIDBASE for all data addresses causes a design 	*/
/* problem in -pid mode: Zero Data Area (absolutely located)		*/
/* Optimization (.zbss/.data) is not implemented.			*/
/* If the symbol RODATA_IS_ABSOLUTE is defined,  then PICBASE is not    */
/* added for read-only data items.    This define can be set if the     */
/* compiler is used in a mode where read-only data is used even when    */
/* PID and/or PID features are used as well.    Green Hills debug	*/
/* servers can avoid relocating read-only data sections when the        */
/* ABS directive is used in section directives files to specify that a  */
/* a read-only data section should always be downloaded/located at its  */
/* link-time address.							*/
/*									*/
/* For absolute relocation, the .secinfo pointers are all absolute, and	*/
/* no special pointer relocation is necessary.  For applications which	*/
/* need relocation for some sections but not others, you can customize	*/
/* the .secinfo data or customize the code in this startup module.	*/
/*									*/
/* More arguments to the initialization routine:  argc, argv, and envp.	*/
/* These are valid if the program is run from the debugger or 0 if not.	*/
/* If argc==0, we construct the arguments to main() on our stack frame. */
/* You may change these arguments to provide any other default values	*/
/*----------------------------------------------------------------------*/

#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include "inddef.h"
#include "ind_crt0.h"
#include "ind_exit.h"
#include "ind_thrd.h"

#pragma ghs nopicpidfixups /* remove when compiler sets .fixtype weak bit */
#pragma ghs Z1715	   /* disable argument checking for main() */


#pragma weak	  __ghsbegin_picbase
#pragma weak	  __ghsbegin_robase
#pragma weak	  __ghsbegin_pidbase

char **environ;
extern void	  __ghsbegin_picbase(void);	/* moves with .text	*/
extern const char __ghsbegin_robase[];		/* moves with .rodata	*/
extern char	  __ghsbegin_pidbase[];		/* moves with .data	*/

#define CONST_FUNCP *const

#ifdef RODATA_IS_ABSOLUTE

  /* rodata is absolute */
  typedef const char rodata_ptr[];	
# define PIRBASE

#elif defined(RODATA_IS_INDEPENDENT)

# pragma ghs X1024
# pragma ghs X702
# pragma ghs X236
# pragma ghs X1566
  typedef void rodata_ptr(void);		
# define PIRBASE 	BASE[PIC_SEC]
# define CONST_FUNCP *

#else

  /* rodata moves with PIC base */
  typedef void rodata_ptr(void);		
# define PIRBASE 	BASE[PIC_SEC]

#endif	/* RODATA_IS_ABSOLUTE */

typedef enum { ABS_SEC = 0, PIC_SEC, PIR_SEC, PID_SEC } SECTION_BASE;

/* [DDO] Thu Jul 16 11:52:46 1998 */
void __ghs_init_mem(char *baseptrs[])
{
    size_t BASE[4];

    BASE [ABS_SEC]	= 0;
    BASE [PIC_SEC]	= ((char *) __ghsbegin_picbase)	- baseptrs[0];
    BASE [PIR_SEC]	= ((char *) __ghsbegin_robase)	- baseptrs[1];
    BASE [PID_SEC]	= ((char *) __ghsbegin_pidbase)	- baseptrs[2];

#define	PICBASE		BASE[PIC_SEC]
#define	PIDBASE		BASE[PID_SEC]

/******** WARNING:  READ/WRITE GLOBAL VARIABLES ARE NOT YET VALID!  ********/

/*----------------------------------------------------------------------*/
/*									*/
/*	Clear BSS							*/
/*									*/
/*----------------------------------------------------------------------*/

    { /* The .secinfo section is in text; declare functions to force PIC */

    extern rodata_ptr __ghsbinfo_clear, __ghseinfo_clear;

    void **b = (void **) __ghsbinfo_clear; 
    void **e = (void **) __ghseinfo_clear; 

    while (b != e) {
	void *		t;			/* target pointer	*/
	ptrdiff_t	v;			/* value to set		*/
	size_t		n;			/* set n bytes		*/
	t = PIDBASE + (char *)(*b++);
	v = *((ptrdiff_t *) b); b++;
	n = *((size_t    *) b); b++;
	memset(t, v, n);
    }
    }

/*----------------------------------------------------------------------*/
/*									*/
/*	Copy from ROM to RAM						*/
/*									*/
/*----------------------------------------------------------------------*/

    {

    extern rodata_ptr __ghsbinfo_copy, __ghseinfo_copy;

    void **b = (void **) __ghsbinfo_copy;
    void **e = (void **) __ghseinfo_copy;

    while (b != e) {
	void *	t;				/* target pointer	*/
	void *	s;				/* source pointer	*/
	size_t	n;				/* copy n bytes		*/
	t = PIDBASE + (char *)(*b++);
	s = PIRBASE + (char *)(*b++);
	n = *((size_t *) b); b++;
	memcpy(t, s, n);
    }
    }

/*----------------------------------------------------------------------*/
/*									*/
/*	V830 block copy of ROM to Internal RAM				*/
/*									*/
/*----------------------------------------------------------------------*/

#if defined(__V830)||defined(__V830R) /* any target with unaligned initialized ptrs */
    /* the following code block is specific to V830 processor;  for all
     * others,  the weak reference to the iramcpy function will be 0.
     * For the V830,  this function copies from ROM to built-in
     * instruction RAM (sections with ISROM shadow copies in section maps)
     */

    {
    #pragma weak iramcpy

    extern void *iramcpy(void *dest_addr, void *src_addr, size_t n);
    static void *(CONST_FUNCP iramcpy_funcp)(void *, void *, size_t) = iramcpy;

	if (iramcpy_funcp) {
	    extern rodata_ptr __ghsbinfo_bcopy, __ghseinfo_bcopy;

	    void **b = (void **)__ghsbinfo_bcopy;
	    void **e = (void **)__ghseinfo_bcopy;
	    while (b < e) {
		void *s, *ct, *n;
		s  = PIDBASE + (char *)(*b++);
		ct = PIRBASE + (char *)(*b++);
		n = *b++;
		iramcpy((char *)s, ct, ((size_t) n + 15) & ~15);
	    }
	}
    }
#endif

/*----------------------------------------------------------------------*/
/* WARNING: LOTS OF SUBTLE IMPLEMENTATION DEFINED STUFF HAPPENS HERE.	*/
/*									*/
/* This code relocates initialized pointers in PIC, PID, or PIR modes.	*/
/* (PIC, PIR, PID = Position-Independent Code, Read-Only Data, Data.)	*/
/* The Green Hills implementation of PIC, PIR, and PID assumes each	*/
/* address is a runtime address when used, not a linktime address.	*/
/* The compiler emits code to add PICBASE, PIRBASE, or PIDBASE when an	*/
/* address is taken (C '&' operator) and does not emit code when an	*/
/* address is dereferenced (C '*' or '[]' operators).  This is called	*/
/* early binding, as opposed to late binding.  If a global or static	*/
/* variable is initialized with	an expression containing '&', the C	*/
/* program expects this variable to have the correct value by the time	*/
/* main() is called.  Version 1.8.8 or later Green Hills compilers and	*/
/* ind_crt0.c work together to make this assumption true.		*/
/*									*/
/* How it works: When the compiler outputs an initializer for a pointer	*/
/* to a position independent object, it also outputs one element each	*/
/* to two Green Hills reserved sections named .fixtype and .fixaddr.	*/
/* For each initialized pointer, there is a corresponding element in	*/
/* .fixaddr and and a corresponding element in .fixtype.		*/
/*									*/
/* Each element in .fixtype is a byte.  Each element in .fixaddr is a	*/
/* 32 or 64 bit pointer of type char *.   Each element in .fixaddr	*/
/* contains the link-time byte address of its corresponding initialized	*/
/* pointer.  Each initialized pointer has TWO associated bases:  the	*/
/* base of the location of the pointer itself (the pointer base) and	*/
/* the base of the location of the object pointed to (the object base).	*/
/*									*/
/* Here	is a picture of a fixtype entry, where XXXX = unused:		*/
/*									*/
/*	 1 bit  1 bit	 2 bits   1 bit  1 bit   2 bits			*/
/*	+------+------------------------------------------+		*/
/*	| Weak | XXXX | Ptr Base | Word | XXXX | Obj Base |		*/
/*	+------+------------------------------------------+		*/
/*									*/
/* The following values are synchronized with compiler internals:	*/
/*									*/
#define FIXTYPE_WEAK	(1<<7)	/* initialized pointer -> weak symbol	*/
#define FIXTYPE_WORD	(1<<3)	/* initialized pointer is a word ptr	*/
/*									*/
/* The Weak bit means the object pointed to is a weak external.		*/
/* The Word bit means the pointer is a word pointer not a byte pointer.	*/
/*									*/
/* The 2 bit Ptr Base and Obj Base fields have the same encoding:	*/
/*									*/
/*	0 ABSBASE	.zdata						*/
/*	1 PICBASE	.text (in -pic mode)				*/
/*	2 PIRBASE	.rosdata, .rodata (PowerPC only)		*/
/*	3 PIDBASE	.sdata, .sbss, .data, .bss (in -pid mode)	*/
/*									*/
/* The linker concatenates sections together and also preserves their	*/
/* input order, so in the executable file there are two sections named	*/
/* .fixtype and .fixaddr.  Using the two 4 bit nibbles in .fixtype, the	*/
/* following code adds the appropriate base to each element of .fixaddr	*/
/* to find each initialized pointer, and then adds the appropriate base	*/
/* to the initialized pointer itself to give it the proper value.	*/
/*----------------------------------------------------------------------*/

#pragma weak __ghsbegin_fixtype
#pragma weak __ghsend_fixtype

#pragma weak __ghsbegin_fixaddr
#pragma weak __ghsend_fixaddr

#pragma weak __ghs_checksum

  {
#if defined(__mips)		/* any target with unaligned initialized ptrs */
	#pragma align(1)
	typedef struct {ptrdiff_t UNAL_PTR; } INIT_PTR;	/* unaligned init ptr */
	#pragma align()
#else
	typedef struct {ptrdiff_t UNAL_PTR; } INIT_PTR;	/*   aligned init ptr */
#endif

    extern rodata_ptr __ghsbegin_fixaddr, __ghsend_fixaddr;
    extern rodata_ptr __ghsbegin_fixtype, __ghsend_fixtype;
    extern char __ghs_checksum[];		/* linker checksum size */

    char **ppaddr	= (char **)	__ghsbegin_fixaddr;
    char **ppaddrend	= (char **)	__ghsend_fixaddr;
    char  *ptype	= (char  *)	__ghsbegin_fixtype; /* -> .fixtype  */

/* The lx -checksum option appends a 32 bit checksum to each non-empty      */
/* section.  If there is no checksum word. lx sets symbol __ghs_checksum    */
/* to 0.  If the section is not empty, we must subtract the checksum word   */
/* size.  Since we're declaring the linker defined symbol as a data item    */
/* here, we must also subtract off the PID offset the compiler		    */
/* automatically inserts.						    */

    if (ppaddrend != ppaddr)
        ppaddrend = (char **)((size_t)ppaddrend-
			      ((size_t)__ghs_checksum-PIDBASE));

    while (ppaddr != ppaddrend) {
	ptrdiff_t	tmp;
	ptrdiff_t	Base;
	INIT_PTR*	ptr_to_init_ptr;	/* .fixaddr entry ->	*/
	int type = *ptype++;			/* next .fixtype byte	*/

#if defined(__68K) && defined(__COFF)
	while (type == 0) type = *ptype++;	/* l68 adds 0 bytes	*/
#endif

	Base = (ptrdiff_t) BASE[(type & (3<<4)) >> 4];	/* Byte Base	*/
	ptr_to_init_ptr = (INIT_PTR *)(Base + *ppaddr++);

	tmp = ptr_to_init_ptr->UNAL_PTR;	/* init ptr may be unaligned  */

	if (((type & FIXTYPE_WEAK) == 0) ||	/* if symbol NOT weak	*/
	       (tmp != 0))	    {		/* or symbol NOT 0 then	*/
						/* reloc initialize ptr	*/
	    Base = (ptrdiff_t) BASE[type & 3];	/* get Byte Base	*/
	    if (!Base) continue;		/* address already ok   */

	    if (type & FIXTYPE_WORD) tmp<<=2;	/* Word Ptr => Byte Ptr	*/
	    tmp += Base;			/* Add Byte Base	*/
	    if (type & FIXTYPE_WORD) tmp>>=2;	/* Byte Ptr => Word Ptr	*/

	    ptr_to_init_ptr->UNAL_PTR = tmp;	/* init ptr may be unaligned */
	}
    }
  }
}

/*----------------------------------------------------------------------*/
/*									*/
/*	Entry Point __ghs_ind_crt0					*/
/*									*/
/*----------------------------------------------------------------------*/

void __ghs_ind_crt0 (char *baseptrs[], int argc, char *argv[], char *envp[])
{
    /* [DDO] Thu Jul 16 11:52:46 1998 */
    __ghs_init_mem(baseptrs);
    
    /*** RAM SECTIONS ARE NOW FULLY INITIALIZED ***/
    /* IT IS NOW SAFE TO USE ARBITRARY GLOBAL VARIABLES */
    
/*---------------------*/
/* initialize ind_io.c */
/*---------------------*/
    _enter();				/* initialize ind_io.c */

/*-----------------*/
/* initialize lock */
/*-----------------*/
    {
    #pragma weak __gh_lock_init
    extern void __gh_lock_init(void);
    static void (CONST_FUNCP lock_init_funcp)(void) = __gh_lock_init;
    /* If ind_thrd.c is loaded, initialize the C library lock */
    if (lock_init_funcp) __gh_lock_init();
    }
	
/*----------------*/
/* initialize iob */
/*----------------*/
    {
    #pragma weak iob_init
    extern void iob_init(void);
    static void (CONST_FUNCP iob_init_funcp)(void) = iob_init;
    /* if cciob.c is loaded, initialize _iob for stdin,stdout,stderr */
    if (iob_init_funcp) iob_init();
    }

/*------------------*/
/* initialize error */
/*------------------*/
    {
    #pragma weak __gh_error_init
    extern void __gh_error_init(void);
    static void (CONST_FUNCP error_init_funcp)(void) = __gh_error_init;
    /* if indrnerr.c is loaded, initialize error */
    if (error_init_funcp) __gh_error_init();
    }

/*-------------------*/
/* initialize signal */
/*-------------------*/
    {
    #pragma weak __gh_signal_init
    extern void __gh_signal_init(void *);
    static void (CONST_FUNCP signal_init_funcp)(void *) = __gh_signal_init;
    /* if ind_sgnl.c is loaded, initialize the signal functions array */
    if (signal_init_funcp) __gh_signal_init(0);
    }

#ifdef __ghs_thread_safe
/*--------------------*/
/* C++ Thread-safe    */
/* Exception handling */
/*--------------------*/
    {
	#pragma weak __cpp_exception_init
	extern void __cpp_exception_init(void **);
	static void (CONST_FUNCP cpp_init_funcp)(void **) = __cpp_exception_init;
	if (cpp_init_funcp) 
	    __cpp_exception_init(&(GetThreadLocalStorage()->__eh_globals));
    }
#endif


#if defined(__i960)
/*-----------------------------------*/
/* special initialization for mon960 */
/*-----------------------------------*/
    {
    /* Initialize the mon960 timers for profiling i960	*/
    /* programs downloaded to mon960 by MULTI/960SERV.	*/
    extern void _startmon960(void);
    _startmon960();			/* defined in file mon960.c */
    }
#endif


    /* New code: bcarnes Tue Oct 21 14:30:38 PDT 1997
     * We now have profiling support on all pmon/mips boards
     * which possess a 2681 DUART */
#if defined(__mips) && !defined(__TRW_RH32__) && !defined(__MIPSX__)
    {
	int __ghs_start_pmon_profiling(void);
	__ghs_start_pmon_profiling();			/* defined below */
    }
#endif	

/*--------------------------------------------*/
/* in ROM mode, set default arguments to main */
/*--------------------------------------------*/
    {

    /* Declare 'main' as a far function on MIPS when in LocalPIC mode,
       so that it can be reached however large the user program is.
       Without being 'far', 'main' can be out of the range of 16-bits
       that a local pic call allows when the user program is too large. */
#if defined(__mips) && !defined(__TRW_RH32) && !defined(__MIPS_X) && \
	defined(__ghs_pic) && defined(__localpic)
#pragma ghs callmode=far
    extern 	int main(int argc, char ** argv, char **envp);
#pragma ghs callmode=default
#else
    extern	int main (int argc, char **argv, char **envp);
#endif

/*  char **environ; */

    /* THESE ARE THE DEFAULT ARGUMENTS TO main() WHEN WE EXECUTE FROM ROM */
    if (!argc) {
	char noname[2];
	char *arg[2];
	char *env[2];

	noname[0] = 0;
	noname[1] = 0;

	arg[0] = noname;
	arg[1] = 0;

	env[0] = noname+1;
	env[1] = 0;

	envp = env;
	argv = arg;

	argc = 1;
    }
    environ = envp;

/*------------------------------*/
/* call main(argc, argv, envp)	*/
/*------------------------------*/
    /* From libsys we don't call exit() which is in a higher level library. */
    /* Instead, we call _exit() which is in this library.  There are a few  */
    /* things in libansi.a which may need to be called by _exit().  This is */
    /* handled by calling __ghs_at_exit() from libansi.a to setup the calls */
    _exit(main(argc, argv, envp));	/* _exit() will shut down ind_io.c  */
    /* _exit() should never return. If it does, let our caller handle it.   */
    return;
    }
}
