/*
		Language Independent Library

	Copyright 1983-1996 Green Hills Software,Inc.

    This program is the property of Green Hills Software, Inc,
    its contents are proprietary information and no part of it
    is to be disclosed to anyone except employees of Green Hills
    Software, Inc., or as agreed in writing signed by the President
    of Green Hills Software, Inc.
*/

/*
 * As a help to making the C library thread-safe, all references to errno
 * are done through these 2 functions, at least when EMEDDED is defined.
*/
#include "inddef.h"
#include "inderrno.h"
#include "ind_thrd.h"

/*
 * In a multi-process environment, these modified functions demonstrate at
 * least one way to maintain a separate copy of errno for each process.
 * It is assumed that some form of lock is available.
*/
#ifdef MULTI_PROCESS

#undef __gh_set_errno
#undef __gh_get_errno
#ifndef MAXIMUM_PROCESSES
#define MAXIMUM_PROCESSES 16
#endif
static struct s { int id, errno; } list[MAXIMUM_PROCESSES];
static int num_proc;

static int *get_errno_ptr(void)
{
    int i, pid;
    struct s *p;
    int getpid(void);

    /* lock the semaphore */
    pid = getpid();
    for (i=0, p=list; i < num_proc; i++)
	if (p->id == pid)
	    return &p->errno;
    if (i < MAXIMUM_PROCESSES) {
	num_proc = i;
	p->id = pid;
	return &p->errno;
    }
    else return (void*)0;
}
void __gh_set_errno(int err)
{
    int *p;
    p = get_errno_ptr();
    if (p) *p = err;
    /* unlock the semaphore */
}
int  __gh_get_errno(void)
{
    int err;
    int *p;
    p = get_errno_ptr();
    if (p) err = *p;
    else   err = 0;
    /* unlock the semaphore */
    return err;
}
#else
int *__gh_errno_ptr(void)
{
    ThreadLocalStorage* tlsptr = GetThreadLocalStorage();

    return &tlsptr->Errno;
}
/* [ciaccio] Fri Jan 16 12:07:23 1998 if we are not multi-threaded then we should
   still use the global threadlocalstorage structure defined in ind_thrd.c which
   was meant to replace the global errno.     old code:
void __gh_set_errno(int err)
{
    errno = err;
}

int  __gh_get_errno(void)
{
    return errno;
}
  end old code, new code: */

void __gh_set_errno(int err)
{
    int *p;
    p = __gh_errno_ptr();
    if (p) *p = err;
}

int  __gh_get_errno(void)
{
    int err;
    int *p;
    p = __gh_errno_ptr();
    if (p) err = *p;
    else   err = 0;
    return err;
}
/* [JY] Thu Oct 30 23:20:11 PST 1997.  Normal programs which are not compiled
    with the thread-safe option will have unresolved references to errno.
    We need to provide a definition of errno for such programs.  I know of no
    way to determine whether the end-user is using the multi-thread feature,
    therefore, I propose to always allocate errno here.  There may be a better
    solution.  If so, change this code and leave an explanatory comment.
*/
#undef errno
int errno;
/* end of new code */
#endif
