/*
		    ANSI C Runtime Library
	
	Copyright 1983-1996 Green Hills Software, Inc.

    This program is the property of Green Hills Software, Inc,
    its contents are proprietary information and no part of it
    is to be disclosed to anyone except employees of Green Hills
    Software, Inc., or as agreed in writing signed by the President
    of Green Hills Software, Inc.
*/
/* ind_gprf.c: Machine Independent Call Graph Profiling module. */

#include "inddef.h"
#include "ind_crt0.h"
#include "ind_io.h"
#include "ind_exit.h"
#include "ind_heap.h"

/*[TPW] Do only .text for the moment, need to add .secinfo driven code. */
extern void __ghsbegin_text(void), __ghsend_text(void);

typedef struct {
    int	pc;
    int	count;
} CNT;

static CNT *__arcbuf__;
static int narcs; 
#ifndef __ghs_thread_safe
static int in_gcount;
#endif

/*[TPW] ARCBUCKETSIZE is the number of text bytes per arc bucket, and should be
	equal to the minimum spacing between subroutine calls for a given
	architecture.

	The 68K value is 4, which is true for compiler generated code, as the
	smallest subroutine call will be a BSR.W which is 4 bytes. It is
	physically possible to have 2 byte spaced subroutine calls, by using
	BSR.S in assembly modules to call compiler generated subroutines, but
	profiling is bound to get mighty confused in such a situation anyway.
*/
#if (defined(__FR20) || defined(__MCORE__))
#define ARCBUCKETSIZE 2
#else
#define ARCBUCKETSIZE 4
#endif

#ifdef __Long_Is_32
typedef unsigned long U32;
#else
#ifdef __Int_Is_32
typedef unsigned int U32;
#else
#error I need a 32 bit type.  recode me
#endif
#endif

#define W_ERROR(s) write(2, s, sizeof(s)-1);
#define P_ERROR(msg) do { static const char s[] = msg; W_ERROR(s); } while (0)

/* the "count" field in the gmon.out header appears to really be the offset
	into the file at which the arc buckets start. It works for me...
*/
static void _stopmon(void)
{
    int fd;
    int h[3];		/* lpc, hpc, count */
    static const char gmonout[] = "gmon.out";

#ifndef __ghs_thread_safe
    in_gcount = 1;		/* disable profiling */
#endif

    h[0] = h[1] = 0;		/* no sampling information */
    h[2] = sizeof(h);
    if ((fd = creat(gmonout, 0660)) < 0) {
	P_ERROR("gcount: could not create gmon.out\n");
    } else {
        static const char writefailed[] = "gcount: could not write gmon.out\n";
	if (write(fd, h, sizeof(h)) != sizeof(h)) {
	    W_ERROR(writefailed);
	} else {
	    CNT *p = __arcbuf__;
	    int n = narcs;
	    unsigned int r[3];		/* frompc, selfpc, count */

	    r[0] = (unsigned int) __ghsbegin_text;
	    for ( ; n--; r[0] += ARCBUCKETSIZE, ++p) {
		U32 l;
		if (0 == (l = p->count))
		    continue;
		r[2] = l;
		r[1] = (unsigned int) p->pc;
		if (write(fd, r, sizeof(r)) != sizeof(r)) {
		    W_ERROR(writefailed);
		    break;
		}
	    }
	}
	close(fd);
    }
    /* ideally should free the buffers, but __ghs_alloc() doesn't allow that. */
    /* do not clear in_gcount, because we are finished profiling. */
}

#define UDIVBYARCBS(x) ((U32)(x)/ARCBUCKETSIZE)

void __ghs_indgcount(char *archead, char **parctail)
{
    unsigned int arcs;
    CNT *this;
    static int nomemory;

#ifndef __ghs_thread_safe
    if (in_gcount)
	return;
    in_gcount = 1;
#endif
    if (nomemory)
	return;

    if (!__arcbuf__) {
	static struct __GHS_AT_EXIT gae;
	/* round up to make sure we have enough buckets */
	narcs = UDIVBYARCBS((char *)__ghsend_text - (char *)__ghsbegin_text
			+ ARCBUCKETSIZE-1);
	__arcbuf__ = (CNT *) __ghs_calloc(sizeof(CNT)*narcs, 16);
	if (!__arcbuf__) {
	    nomemory = 1;
	    P_ERROR("gcount: could not allocate memory\n");
	    return;		/* LEAVE in_gcount SET, we're inactive now. */
	} else {
	    gae.func = _stopmon;
	    __ghs_at_exit(&gae);
	}
    }
    this = __arcbuf__;
    arcs = UDIVBYARCBS(*parctail - (char *)__ghsbegin_text);	/* round down */
    if (arcs >= narcs)
	return;		/* avoid trashing memory in multiple section case */
    this += arcs;
    this->count++;
    this->pc = (int) archead;

#ifndef __ghs_thread_safe
    in_gcount = 0;
#endif
}
