/*
		C++ Library

	Copyright 1983-1996 Green Hills Software,Inc.

    This program is the property of Green Hills Software, Inc,
    its contents are proprietary information and no part of it
    is to be disclosed to anyone except employees of Green Hills
    Software, Inc., or as agreed in writing signed by the President
    of Green Hills Software, Inc.
*/

/*
 * Implementation of the C++ library locks
 */

/* Take a lock specified by the parameter passed in. A thread can return from
 * this function only if no other thread has called it with with the same
 * value of 'i' without releasing the lock. Currently, the values of 'i' are
 * in the range [0,2], inclusive.  See 'yvals.h' in the ecxx, eecxx, scxx
 * directories.
 */

#if defined(__Chorus)

/*
 * Chorus does not provide a recursive mutex which is required by 
 * the C++ libraries.  We synthesize one here.
 * DO NOT CHANGE THE ORDER OF ANY STATEMENTS in __ghs{Unl,L}ockCpp()
 */

#include <sync/chMutex.h>
#include <exec/chExec.h>

struct RecursiveMutex {
    KnMutex	mutex;
    KnThreadLid	owner;
    int		count;
};

#define THREAD_ID_NIL	(0)

struct RecursiveMutex theLocks[3] = {
	{ K_KNMUTEX_INITIALIZER, THREAD_ID_NIL, 0},
	{ K_KNMUTEX_INITIALIZER, THREAD_ID_NIL, 0},
	{ K_KNMUTEX_INITIALIZER, THREAD_ID_NIL, 0} 
	};

void __ghsLockCpp(int i)
{
    struct RecursiveMutex * rm = theLocks+i;
    KnThreadLid myThreadId = threadSelf();
    
    if(rm->owner==myThreadId) {
	rm->count++;
    } else {
	mutexGet(&rm->mutex);
	rm->owner = myThreadId;
	rm->count = 1;
    }
}

void __ghsUnlockCpp(int i)
{
    struct RecursiveMutex * rm = theLocks+i;
    if(--(rm->count)==0) {
	rm->owner = THREAD_ID_NIL;
	mutexRel(&rm->mutex);
    }
}

#else /* __Chorus */

#include "ind_thrd.h"

void __ghsLockCpp(int i) 
{ 
    __ghsLock();
}

void __ghsUnlockCpp(int i) 
{ 
    __ghsUnlock();
}


#endif 
