/*
		    ANSI C Runtime Library
	
	Copyright 1983-1996 Green Hills Software, Inc.

    This program is the property of Green Hills Software, Inc,
    its contents are proprietary information and no part of it
    is to be disclosed to anyone except employees of Green Hills
    Software, Inc., or as agreed in writing signed by the President
    of Green Hills Software, Inc.
*/
/* ind_mprf.c: Machine Independent Call Count Profiling module. */

#include "inddef.h"
#include "ind_exit.h"
#include "ind_heap.h"
#include "ind_io.h"
#include "ind_thrd.h"

/* each mcount buffer chunk is CNTCHUNKSIZE CNT's, plus one for management */
#define CNTCHUNKSIZE 1024

typedef struct {
    int	pc;
    int	count;
} CNT;

static CNT __cnthead__, *__nextcnt__;
static int ncnt;
#ifndef __ghs_thread_safe
static int in_mcount;
#endif

#define W_ERROR(s) write(2, s, sizeof(s)-1);
#define P_ERROR(msg) do { static const char s[] = msg; W_ERROR(s); } while (0)

static void _stopmon(void)
{
    int h[3];		/* lpc, hpc, count */
    int fd, count = 0, *area;
    CNT *c, *d;
    static const char monout[] = "mon.out";

    /* creat() and write() may have been compiled for profiling */
#ifndef __ghs_thread_safe
    in_mcount = 1;
#endif

    /* Replace the pointers in the mcount buckets with the actual mcounts.  */
    /* Count how many there were, and store the count for each chunk in the */
    /* count field of the last CNT (d) in the _previous_ buffer chunk.	    */
    for (d = &__cnthead__; c = (CNT *) d->pc; d = c) {
	for ( ; area = (int *) c->count; ++c)
	    c->count = *area;
	count += (d->count = c - (CNT *) d->pc);
    }

    /* construct header for mon.out file */
    h[0] = h[1] = 0;		/* no stochastic samples */
    h[2] = count;

    /* Write out the file. Walk the list of buffer chunks and write each one. */
    if ((fd = creat(monout, 0660)) < 0) {
        P_ERROR("mcount: could not create mon.out\n");
    } else {
	static const char writefailed[] = "mcount: could not write mon.out\n";
	if (write(fd, h, sizeof(h)) != sizeof(h)) {
	    W_ERROR(writefailed);
	} else {
	    for (c = &__cnthead__; c->pc; c = (CNT *) (c->pc + count)) {
		count = c->count*sizeof(CNT);
		if (write(fd, (void *) c->pc, count) != count) {
		    W_ERROR(writefailed);
		    break;
		}
	    }
	}
	close(fd);
    }
    /* ideally should free the buffers, but __ghs_alloc() doesn't allow that. */
    /* do not clear in_mcount, because we are finished profiling. */
}

void __ghs_indmcount(int *area, char *archead)
{
    static int nomemory;
#ifndef __ghs_thread_safe
    /* This is not thread-safe; our intent is to prevent infinite recursion. */
    if (in_mcount)
	return;
    in_mcount = 1;
#endif

    if (nomemory)
	return;
    if (!__nextcnt__) {
	__nextcnt__ = &__cnthead__;
    }
    if (!*area) {
	__ghsLock();
	if (!ncnt) {
	    void *buf = __ghs_calloc(sizeof(CNT)*(CNTCHUNKSIZE+1), 16);
	    if (!buf) {
		P_ERROR("mcount: could not allocate memory\n");
		__ghsUnlock();
		nomemory = 1;
		return;		/* LEAVE in_mcount SET, we're inactive now. */
	    } else {
		static struct __GHS_AT_EXIT gae;
		/* Jim Thu Apr  1 12:03:41 EST 1999  Don't add this
		 * function if it has already been added.  New Code: */
		if (!gae.func) {
		/* End New Code */
		    gae.func = _stopmon;
		    __ghs_at_exit(&gae);
		/* Jim Thu Apr  1 12:03:41 EST 1999  New Code: */
		}
		/* End New Code */
	    }
	    __nextcnt__->pc = (int) buf;
	    __nextcnt__ = (CNT *) __nextcnt__->pc;
	    ncnt = CNTCHUNKSIZE;
	}
	__nextcnt__->pc = (int) archead;
	__nextcnt__->count = (long) area;
	++__nextcnt__;
	--ncnt;
	__ghsUnlock();
    }
    ++*area;

#ifndef __ghs_thread_safe
    in_mcount = 0;
#endif
}
