/*
		    ANSI C Runtime Library
	
    Copyright 1983,1984,1985,1986,1987,1988,1989,1990,1991,1992,1993,1994
		    Green Hills Software, Inc.

    This program is the property of Green Hills Software, Inc,
    its contents are proprietary information and no part of it
    is to be disclosed to anyone except employees of Green Hills
    Software, Inc., or as agreed in writing signed by the President
    of Green Hills Software, Inc.
*/

#if defined(EMBEDDED)

#include <string.h>

/* heavily unrolled loop, using up to 16-byte alignment (for i960) */
/* this routine is used by gprof and crt0.o */

#if defined(__i960)
# define USE_STRUCT16
# define POWER	5
# define LOOP	(1<<POWER)
#else
# define POWER	4
# define LOOP	(1<<POWER)
#endif

#pragma ghs nosmallandslow

#if defined(__V800)

/* [JY] I reversed the order of the writes, since we have to remember the base*/
/* or the block.  Figure out the end of the block and work backwards.  Needs  */
/* only one active pointer, plus the saved base pointer, and no counters.     */
/* Old way _may_ be better on 68k, with its cisc logic, but maybe not. */

void *memset(void *s, int c, size_t n) {
    unsigned char *pt = (unsigned char*)s + n;
    unsigned char ch=c; 
    unsigned char *stop1 = (unsigned char*)s-1, *stop2;

    if (n > 2*LOOP) {
	/* we have at least one aligned 16 byte block */
	register int ch4;

	/* do this first because it will use several temporary registers */
	ch4 = ch;
	ch4 <<= 8;
	ch4 |= ch;
	ch4 = (ch4 << 16 | ch4);

	/* get rid of odd bytes at the end first, and stop using n */
	while ((size_t)pt & LOOP-1)
	    *--pt = ch;

	/* pt is now aligned, pointing above the top block */

	{
#ifdef USE_STRUCT16
	    register struct quad { int b1,b2,b3,b4; } ch16;
	    ch16.b1=ch4; ch16.b2=ch4; ch16.b3=ch4; ch16.b4=ch4;
#endif
	    /* [jonhsu] Mon Jun 29 11:38:05 PDT 1998  The usage of the simple
	       comparison between pt and s doesn't account for when s is
	       at or below zero while pt is above zero.
	    New code: */
	    stop2 = (unsigned char*)((size_t)s&(~(LOOP-1)));
	    /* End new code */
/*[TPW/edlee] Thu Feb 29 00:51:24 PST 1996.
	Jeff's turbo memset() enhancement failed to compensate for overshoot
	on a pointer it was walking backwards through the memory block. Add
	one instruction after the loop to correct this. Also, change his
	termination test to better optimize the case where the start of the
	memory block is favorably aligned. 2 places.
** Old Code: **
	    while ((pt -= LOOP) > s) {
** New Code: */
	    /* [jonhsu] Mon Jun 29 11:38:05 PDT 1998  The usage of the simple
	       comparison between pt and s doesn't account for when s is
	       at or below zero while pt is above zero.
	    Old code:
	    while ((pt -= LOOP) >= s) {
	    New code: */
	    while ((pt -= LOOP) != stop2) {
	    /* End new code */
/* End of New Code */
#ifdef USE_STRUCT16
		((struct quad*)pt)[0] = ch16;
		((struct quad*)pt)[1] = ch16;
#else
		((int *)pt)[0] = ch4;
		((int *)pt)[1] = ch4;
# if POWER >= 4
		((int *)pt)[2] = ch4;
		((int *)pt)[3] = ch4;
#  if POWER >= 5
		((int *)pt)[4] = ch4;
		((int *)pt)[5] = ch4;
		((int *)pt)[6] = ch4;
		((int *)pt)[7] = ch4;
#  endif
# endif
#endif
	    }
/*[TPW/edlee] Thu Feb 29 00:51:24 PST 1996.
	Jeff's turbo memset() enhancement failed to compensate for overshoot
	on a pointer it was walking backwards through the memory block. Add
	one instruction after the loop to correct this. Also, change his
	termination test to better optimize the case where the start of the
	memory block is favorably aligned. 2 places.
** New Code: */
	    pt += LOOP;
/* End of New Code */
	}
    }
    /* [jonhsu] Mon Jun 29 11:38:05 PDT 1998  The usage of the simple
       comparison between pt and s doesn't account for when s is
       at or below zero while pt is above zero.
    Old code:
    while (--pt >= s)
    New code: */
    while (--pt != stop1)
    /* End new code */
	*pt = ch;
    
    return s;
}

#else

/* Todd's original fast routine */

void *memset(void *s, int c, size_t n) {
    register unsigned char *pt=s;
    unsigned char ch=c; 

    if (n > 2*LOOP) {
	/* we have at least one aligned 16 byte block */
	register int i;
	register int ch4;
	register struct quad { int b1,b2,b3,b4; } *pts;

	/* do this first because it will use several temporary registers */
	ch4 = ch;
	ch4 <<= 8;
	ch4 |= ch;
	ch4 = (ch4 << 16 | ch4);

	i = LOOP - ((size_t)pt & (LOOP-1)); /* i is bytes before the blocks */

	if (i) {
	    n -= i;			/* n is bytes in blocks and after blocks */
	    do {
		*pt = ch;
		pt++;
	    } while (--i);
	}
	/* we are now at an block boundary. */

	i = n >> POWER;		/* i is number of blocks */
	n &= LOOP-1;		/* n is bytes after the block */

	{
#ifdef USE_STRUCT16
	    register struct quad ch16;
	    ch16.b1=ch4; ch16.b2=ch4; ch16.b3=ch4; ch16.b4=ch4;
#endif
	    do {
		/* loop body is 8 instructions, loop overhead is 3 instructions */
#ifdef USE_STRUCT16
		((struct quad*)pt)[0] = ch16;
		((struct quad*)pt)[1] = ch16;
		pt += 2 * sizeof(struct quad);
#else
		((struct quad*)pt)[0].b1 = ch4;
		((struct quad*)pt)[0].b2 = ch4;
		((struct quad*)pt)[0].b3 = ch4;
		((struct quad*)pt)[0].b4 = ch4;
#if POWER == 4
		pt += sizeof(struct quad);
#else
		((struct quad*)pt)[1].b1 = ch4;
		((struct quad*)pt)[1].b2 = ch4;
		((struct quad*)pt)[1].b3 = ch4;
		((struct quad*)pt)[1].b4 = ch4;
		pt += 2 * sizeof(struct quad);
#endif
#endif
	    } while (--i);
	}
    }
    if (n)	/* n is bytes after the blocks */
	do {
	    *pt = ch;
	    pt++;
	} while ( --n );
    return(s);
}
#endif

#endif /* defined(EMBEDDED) */
