/*
		      ANSI C Runtime Library

	Copyright (c) 1983-1997 Green Hills Software, Inc.

	This program is the property of Green Hills Software, Inc. Its contents
	are proprietary information and no part of it is to be disclosed to
	anyone except employees of Green Hills Software, Inc., or as agreed
	in writing signed by the President of Green Hills Software, Inc.
*/
/*
    ANSI C requires these functions, and allows them to also be macros.
    ANSI C++ requires them to be only functions, but allows them to be inline.

    This implementation works as follows:
	
    1.	In C, all functions are prototyped unconditionally.
	In C++, all functions are prototyped unless __CALL_CTYPE_FUNCTIONS__
    2.	If defined __CALL_CTYPE_FUNCTIONS__, only prototypes are provided for
	the functions and the rest of the file is skipped.  No macros 
	will be defined and no inline functions will be declared.
    3.	Inline versions of all functions are declared with hidden names.
	(In C, this depends on the Green Hills C __inline feature.)
    4.	In C++, inline versions of the functions are declared which invoke
	the inline versions with hidden names.  If the address of isspace is
	taken, a static copy of the function will be created.
    5.	In C, macros are define'd which invoke the inline versions with hidden
	names.  To take the address of isspace, first use #undef isspace to 
	remove the macro, then take the address of isspace.
    6.	In all cases, these functions are safe:
	a.  Arguments with side-effects are always evaluated once.
	b.  Out of range arguments always return false.
*/
	 
#ifndef _GHS_CTYPE_H
#define _GHS_CTYPE_H

#ifdef __cplusplus
extern "C" {
# if !defined(__inline)
#  define __inline inline
# endif	/* __inline */
#endif	/* __cplusplus */

#if defined(__STDC__) || defined(__PROTOTYPES__)
/* ANSI C, C++ style function arguments */
# define __A0	int c
# define __A1	int c
# define __A2
#else
/* K&R style function arguments */
# define __A0
# define __A1	c
# define __A2	int c;
#endif	/* __STDC__ */

/* Prototypes are required in ANSI always.  
   Prototypes are needed in C++ when inline versions are not used. */
#if !defined(__cplusplus) || defined(__CALL_CTYPE_FUNCTIONS__)
int isalnum (__A0);
int isalpha (__A0);
int iscntrl (__A0);
int isdigit (__A0);
int isgraph (__A0);
int islower (__A0);
int isprint (__A0);
int ispunct (__A0);
int isspace (__A0);
int isupper (__A0);
int isxdigit(__A0);
int tolower (__A0);
int toupper (__A0);
#if __STDC__==0 && !defined(_POSIX_SOURCE) && !defined(__VXWORKS)
int isascii (__A0);
int toascii (__A0);
#endif	/* non-ANSI C functions    */
#endif /* defined(__cplusplus) || defined(__CALL_CTYPE_FUNCTIONS__) */

#ifndef __CALL_CTYPE_FUNCTIONS__

#define	__INRANGE(c,l,h) (((unsigned) ((c) - (l))) <= ((unsigned) ((h)-(l))))

#if defined(__mips)
# define	_OR_	|		/*  SLTIU is shorter than BLT/NOP */
#else
# define	_OR_	||
#endif	/* __mips */

__inline int __ISALPHA(__A1) __A2 {
  return __INRANGE(((c)|0x20),'a','z');
}
__inline int __ISALNUM(__A1) __A2 {
  return (__INRANGE(((c)|0x20),'a','z') _OR_ __INRANGE(c,'0','9'));
}
__inline int __ISCNTRL(__A1) __A2 {
  return ((((unsigned)(c))<=0x1f) _OR_ ((c)==0x7f));
}
__inline int __ISDIGIT(__A1) __A2 {
  return __INRANGE(c,'0','9');
}
__inline int __ISGRAPH(__A1) __A2 {
  return __INRANGE(c,0x21,0x7e);
}
__inline int __ISLOWER(__A1) __A2 {
  return __INRANGE(c,'a','z');
}
__inline int __ISPRINT(__A1) __A2 {
  return __INRANGE(c,0x20,0x7e);
}
__inline int __ISPUNCT(__A1) __A2 {
  return ((__INRANGE(c,'!','/')) _OR_ (__INRANGE(c,':','@')) _OR_ 
	  (__INRANGE(c,'[','`')) _OR_ (__INRANGE(c,'{','~')));
}
__inline int __ISSPACE(__A1) __A2 {
  return (((c)==0x20) _OR_ __INRANGE(c,'\t','\r'));
}
__inline int __ISUPPER(__A1) __A2 {
  return __INRANGE(c,'A','Z');
}
__inline int __ISXDIGIT(__A1) __A2 {
  return ((__INRANGE(c,'0','9')) _OR_ (__INRANGE(((c)|0x20),'a','f')));
}
__inline int __TOLOWER(__A1) __A2 {
  return (c + (__INRANGE(c,'A','Z')<<5));
/*return (__INRANGE(c,'A','Z') ? 'a'+((c)-'A') : (c));*/
}
__inline int __TOUPPER(__A1) __A2 {
  return (c - (__INRANGE(c,'a','z')<<5));
/*return (__INRANGE(c,'a','z') ? 'A'+((c)-'a') : (c));*/
}

#if __STDC__==0 && !defined(_POSIX_SOURCE) && !defined(__VXWORKS) /* non-ANSI C functions	*/
__inline int __ISASCII(__A1) __A2  {
  return (((unsigned)(c))<=0x7f);
}
__inline int __TOASCII(__A1) __A2 {
  return ((c)&0x7f);
}
#endif	/* non-ANSI C functions    */

#if defined(__cplusplus) || defined(__inline)
__inline int isalnum (int c) { return	__ISALNUM(c); }
__inline int isalpha (int c) { return	__ISALPHA(c); }
__inline int iscntrl (int c) { return	__ISCNTRL(c); }
__inline int isdigit (int c) { return	__ISDIGIT(c); }
__inline int isgraph (int c) { return	__ISGRAPH(c); }
__inline int islower (int c) { return	__ISLOWER(c); }
__inline int isprint (int c) { return	__ISPRINT(c); }
__inline int ispunct (int c) { return	__ISPUNCT(c); }
__inline int isspace (int c) { return	__ISSPACE(c); }
__inline int isupper (int c) { return	__ISUPPER(c); }
__inline int isxdigit(int c) { return	__ISXDIGIT(c); }
__inline int tolower (int c) { return	__TOLOWER(c); }
__inline int toupper (int c) { return	__TOUPPER(c); }
#if __STDC__==0 && !defined(_POSIX_SOURCE) && !defined(__VXWORKS) /* non-ANSI C functions	*/
__inline int isascii (int c) { return	__ISASCII(c); }
__inline int toascii (int c) { return	__TOASCII(c); }
#endif	/* non-ANSI C functions    */

#else	/* __cplusplus */

# define isalnum(c) 			__ISALNUM(c)
# define isalpha(c) 			__ISALPHA(c)
# define iscntrl(c)		 	__ISCNTRL(c)
# define isdigit(c)		 	__ISDIGIT(c)
# define isgraph(c) 			__ISGRAPH(c)
# define islower(c) 			__ISLOWER(c)
# define isprint(c) 			__ISPRINT(c)
# define ispunct(c) 			__ISPUNCT(c)
# define isspace(c) 			__ISSPACE(c)
# define isupper(c) 			__ISUPPER(c)
# define isxdigit(c) 			__ISXDIGIT(c)
# define tolower(c) 			__TOLOWER(c)
# define toupper(c) 			__TOUPPER(c)
# if __STDC__==0 && !defined(_POSIX_SOURCE) 	/* non-ANSI C functions	*/
#  define isascii(c) 			__ISASCII(c)
#  define toascii(c) 			__TOASCII(c)
# endif	/* non-ANSI C functions    */
#endif	/* __cplusplus */

#undef __inline
#undef __A1
#undef __A2
#undef __INRANGE

#endif	/* __CALL_CTYPE_FUNCTIONS__ */

#if defined(__cplusplus)
} /* extern "C" */
#endif	/* __cplusplus */

#endif	/* _GHS_CTYPE_H */
