// stl.h supplemental header
#ifndef _STL_H_
#define _STL_H_
#include <algorithm>
#include <deque>
#include <functional>
#include <iterator>
#include <list>
#include <map>
#include <memory>
#include <numeric>
#include <queue>
#include <set>
#include <stack>
#include <utility>
#include <vector>
using namespace std;

#pragma ghs startdata

#ifndef __ghs
/* [nikola] Mon Nov  9 12:22:58 PST 1998 - use the default initializer */
/* Old code */

		// TEMPLATE CLASS Deque
template<class _Ty>
	class Deque : public deque<_Ty, allocator<_Ty> > {
public:
	typedef Deque<_Ty> _Myt;
	typedef allocator<_Ty> _A;
	Deque()
		: deque<_Ty, _A>() {}
	explicit Deque(size_type _N)
		: deque<_Ty, _A>(_N, _Ty()) {}
	Deque(size_type _N, const _Ty& _V)
		: deque<_Ty, _A>(_N, _V) {}
	typedef const_iterator _It;
	Deque(_It _F, _It _L)
		: deque<_Ty, _A>(_F, _L) {}
	};

		// TEMPLATE CLASS List
template<class _Ty>
	class List : public list<_Ty, allocator<_Ty> > {
public:
	typedef List<_Ty> _Myt;
	typedef allocator<_Ty> _A;
	List()
		: list<_Ty, _A>() {}
	explicit List(size_type _N)
		: list<_Ty, _A>(_N, _Ty()) {}
	List(size_type _N, const _Ty& _V)
		: list<_Ty, _A>(_N, _V) {}
	typedef const_iterator _It;
	List(_It _F, _It _L)
		: list<_Ty, _A>(_F, _L) {}
	};

		// TEMPLATE CLASS Map
template<class _K, class _Ty, class _Pr = less<_K> >
	class Map : public map<_K, _Ty, _Pr, allocator<_Ty> > {
public:
	typedef Map<_K, _Ty, _Pr> _Myt;
	typedef allocator<_Ty> _A;
	Map()
		: map<_K, _Ty, _Pr, _A>(_Pr()) {}
	explicit Map(const _Pr& _Pred)
		: map<_K, _Ty, _Pr, _A>(_Pred) {}
	typedef const_iterator _It;
	Map(_It _F, _It _L)
		: map<_K, _Ty, _Pr, _A>(_F, _L, _Pr()) {}
	Map(_It _F, _It _L, const _Pr& _Pred)
		: map<_K, _Ty, _Pr, _A>(_F, _L, _Pred) {}
	};

		// TEMPLATE CLASS Multimap
template<class _K, class _Ty, class _Pr = less<_K> >
	class Multimap
		: public multimap<_K, _Ty, _Pr, allocator<_Ty> > {
public:
	typedef Multimap<_K, _Ty, _Pr> _Myt;
	typedef allocator<_Ty> _A;
	Multimap()
		: multimap<_K, _Ty, _Pr, _A>(_Pr()) {}
	explicit Multimap(const _Pr& _Pred)
		: multimap<_K, _Ty, _Pr, _A>(_Pred) {}
	typedef const_iterator _It;
	Multimap(_It _F, _It _L)
		: multimap<_K, _Ty, _Pr, _A>(_F, _L, _Pr()) {}
	Multimap(_It _F, _It _L, const _Pr& _Pred)
		: multimap<_K, _Ty, _Pr, _A>(_F, _L, _Pred) {}
	};

		// TEMPLATE CLASS Set
template<class _K, class _Pr = less<_K> >
	class Set : public set<_K, _Pr, allocator<_K> > {
public:
	typedef Set<_K, _Pr> _Myt;
	typedef allocator<_K> _A;
	Set()
		: set<_K, _Pr, _A>(_Pr()) {}
	explicit Set(const _Pr& _Pred)
		: set<_K, _Pr, _A>(_Pred) {}
	typedef const_iterator _It;
	Set(_It _F, _It _L)
		: set<_K, _Pr, _A>(_F, _L, _Pr()) {}
	Set(_It _F, _It _L, const _Pr& _Pred)
		: set<_K, _Pr, _A>(_F, _L, _Pred) {}
	};

		// TEMPLATE CLASS Multiset
template<class _K, class _Pr = less<_K> >
	class Multiset : public multiset<_K, _Pr, allocator<_K> > {
public:
	typedef Multiset<_K, _Pr> _Myt;
	typedef allocator<_K> _A;
	Multiset()
		: multiset<_K, _Pr, _A>(_Pr()) {}
	explicit Multiset(const _Pr& _Pred)
		: multiset<_K, _Pr, _A>(_Pred) {}
	typedef const_iterator _It;
	Multiset(_It _F, _It _L)
		: multiset<_K, _Pr, _A>(_F, _L, _Pr()) {}
	Multiset(_It _F, _It _L, const _Pr& _Pred)
		: multiset<_K, _Pr, _A>(_F, _L, _Pred) {}
	};

		// TEMPLATE CLASS Vector
template<class _Ty>
	class Vector : public vector<_Ty, allocator<_Ty> > {
public:
	typedef Vector<_Ty> _Myt;
	typedef allocator<_Ty> _A;
	Vector()
		: vector<_Ty, _A>() {}
	explicit Vector(size_type _N)
		: vector<_Ty, _A>(_N, _Ty()) {}
	Vector(size_type _N, const _Ty& _V)
		: vector<_Ty, _A>(_N, _V) {}
	typedef const_iterator _It;
	Vector(_It _F, _It _L)
		: vector<_Ty, _A>(_F, _L) {}
	};

		// CLASS bit_vector
class bit_vector : public vector<_Bool, _Bool_allocator> {
public:
	typedef _Bool _Ty;
	typedef _Bool_allocator _A;
	typedef bit_vector _Myt;
	bit_vector()
		: vector<_Bool, _Bool_allocator>() {}
	explicit bit_vector(size_type _N, const _Ty& _V = _Ty())
		: vector<_Bool, _Bool_allocator>(_N, _V) {}
	typedef const_iterator _It;
	bit_vector(_It _F, _It _L)
		: vector<_Bool, _Bool_allocator>(_F, _L) {}
	};

		// TEMPLATE CLASS priority_queue
template<class _C,
	class _Pr = less<_C::value_type> >
	class Priority_queue
		: public priority_queue<_C::value_type, _C, _Pr> {
public:
	typedef typename _C::value_type _Ty;
	Priority_queue()
		: priority_queue<_Ty, _C, _Pr>(_Pr()) {}
	explicit Priority_queue(const _Pr& _Pred)
		: priority_queue<_Ty, _C, _Pr>(_Pred) {}
	typedef const _Ty *_It;
	Priority_queue(_It _F, _It _L)
		: priority_queue<_Ty, _C, _Pr>(_F, _L, _Pr()) {}
	Priority_queue(_It _F, _It _L, const _Pr& _Pred)
		: priority_queue<_Ty, _C, _Pr>(_F, _L, _Pred) {}
	};

		// TEMPLATE CLASS queue
template<class _C>
	class Queue
		: public queue<_C::value_type, _C> {
	};

		// TEMPLATE CLASS stack
template<class _C>
	class Stack
		: public stack<_C::value_type, _C> {
	};

#else /* __ghs */
/* New Code */

		// TEMPLATE CLASS Deque
template<class _Ty>
	class Deque : public deque<_Ty> {
public:
	typedef Deque<_Ty> _Myt;
	Deque()
		: deque<_Ty>() {}
	explicit Deque(size_type _N)
		: deque<_Ty>(_N, _Ty()) {}
	Deque(size_type _N, const _Ty& _V)
		: deque<_Ty>(_N, _V) {}
	typedef const_iterator _It;
	Deque(_It _F, _It _L)
		: deque<_Ty>(_F, _L) {}
	};

		// TEMPLATE CLASS List
template<class _Ty>
	class List : public list<_Ty> {
public:
	typedef List<_Ty> _Myt;
	List()
		: list<_Ty>() {}
	explicit List(size_type _N)
		: list<_Ty>(_N, _Ty()) {}
	List(size_type _N, const _Ty& _V)
		: list<_Ty>(_N, _V) {}
	typedef const_iterator _It;
	List(_It _F, _It _L)
		: list<_Ty>(_F, _L) {}
	};

		// TEMPLATE CLASS Map
template<class _K, class _Ty, class _Pr = less<_K> >
	class Map : public map<_K, _Ty, _Pr> {
public:
	typedef Map<_K, _Ty, _Pr> _Myt;
	Map()
		: map<_K, _Ty, _Pr>(_Pr()) {}
	explicit Map(const _Pr& _Pred)
		: map<_K, _Ty, _Pr>(_Pred) {}
	typedef const_iterator _It;
	Map(_It _F, _It _L)
		: map<_K, _Ty, _Pr>(_F, _L, _Pr()) {}
	Map(_It _F, _It _L, const _Pr& _Pred)
		: map<_K, _Ty, _Pr>(_F, _L, _Pred) {}
	};

		// TEMPLATE CLASS Multimap
template<class _K, class _Ty, class _Pr = less<_K> >
	class Multimap
		: public multimap<_K, _Ty, _Pr> {
public:
	typedef Multimap<_K, _Ty, _Pr> _Myt;
	Multimap()
		: multimap<_K, _Ty, _Pr>(_Pr()) {}
	explicit Multimap(const _Pr& _Pred)
		: multimap<_K, _Ty, _Pr>(_Pred) {}
	typedef const_iterator _It;
	Multimap(_It _F, _It _L)
		: multimap<_K, _Ty, _Pr>(_F, _L, _Pr()) {}
	Multimap(_It _F, _It _L, const _Pr& _Pred)
		: multimap<_K, _Ty, _Pr>(_F, _L, _Pred) {}
	};

		// TEMPLATE CLASS Set
template<class _K, class _Pr = less<_K> >
	class Set : public set<_K, _Pr> {
public:
	typedef Set<_K, _Pr> _Myt;
	Set()
		: set<_K, _Pr>(_Pr()) {}
	explicit Set(const _Pr& _Pred)
		: set<_K, _Pr>(_Pred) {}
	typedef const_iterator _It;
	Set(_It _F, _It _L)
		: set<_K, _Pr>(_F, _L, _Pr()) {}
	Set(_It _F, _It _L, const _Pr& _Pred)
		: set<_K, _Pr>(_F, _L, _Pred) {}
	};

		// TEMPLATE CLASS Multiset
template<class _K, class _Pr = less<_K> >
	class Multiset : public multiset<_K, _Pr> {
public:
	typedef Multiset<_K, _Pr> _Myt;
	Multiset()
		: multiset<_K, _Pr>(_Pr()) {}
	explicit Multiset(const _Pr& _Pred)
		: multiset<_K, _Pr>(_Pred) {}
	typedef const_iterator _It;
	Multiset(_It _F, _It _L)
		: multiset<_K, _Pr>(_F, _L, _Pr()) {}
	Multiset(_It _F, _It _L, const _Pr& _Pred)
		: multiset<_K, _Pr>(_F, _L, _Pred) {}
	};

		// TEMPLATE CLASS Vector
template<class _Ty>
	class Vector : public vector<_Ty> {
public:
	typedef Vector<_Ty> _Myt;
	Vector()
		: vector<_Ty>() {}
	explicit Vector(size_type _N)
		: vector<_Ty>(_N, _Ty()) {}
	Vector(size_type _N, const _Ty& _V)
		: vector<_Ty>(_N, _V) {}
	typedef const_iterator _It;
	Vector(_It _F, _It _L)
		: vector<_Ty>(_F, _L) {}
	};

		// CLASS bit_vector
class bit_vector : public vector<_Bool> {
public:
	typedef _Bool _Ty;
	typedef bit_vector _Myt;
	bit_vector()
		: vector<_Bool>() {}
	explicit bit_vector(size_type _N, const _Ty& _V = _Ty())
		: vector<_Bool>(_N, _V) {}
	typedef const_iterator _It;
	bit_vector(_It _F, _It _L)
		: vector<_Bool>(_F, _L) {}
	};

		// TEMPLATE CLASS priority_queue
template<class _C,
	class _Pr = less<_C::value_type> >
	class Priority_queue
		: public priority_queue<_C::value_type, _C, _Pr> {
public:
	typedef typename _C::value_type _Ty;
	Priority_queue()
		: priority_queue<_Ty, _C, _Pr>(_Pr()) {}
	explicit Priority_queue(const _Pr& _Pred)
		: priority_queue<_Ty, _C, _Pr>(_Pred) {}
	typedef const _Ty *_It;
	Priority_queue(_It _F, _It _L)
		: priority_queue<_Ty, _C, _Pr>(_F, _L, _Pr()) {}
	Priority_queue(_It _F, _It _L, const _Pr& _Pred)
		: priority_queue<_Ty, _C, _Pr>(_F, _L, _Pred) {}
	};

		// TEMPLATE CLASS queue
template<class _C>
	class Queue
		: public queue<_C::value_type, _C> {
	};

		// TEMPLATE CLASS stack
template<class _C>
	class Stack
		: public stack<_C::value_type, _C> {
	};

/* End New Code */
#endif /* __ghs */


#pragma ghs enddata

		// MACRO DEFINITIONS
#define deque			Deque
#define list			List
#define map				Map
#define multimap		Multimap
#define set				Set
#define multiset		Multiset
#define vector			Vector
#define priority_queue	Priority_queue
#define queue			Queue
#define stack			Stack

#endif	/* _STL_H_ */

/*
 * Copyright (c) 1996 by P.J. Plauger.  ALL RIGHTS RESERVED. 
 * Consult your license regarding permissions and restrictions.
 */

/*
 * This file is derived from software bearing the following
 * restrictions:
 *
 * Copyright (c) 1994
 * Hewlett-Packard Company
 *
 * Permission to use, copy, modify, distribute and sell this
 * software and its documentation for any purpose is hereby
 * granted without fee, provided that the above copyright notice
 * appear in all copies and that both that copyright notice and
 * this permission notice appear in supporting documentation.
 * Hewlett-Packard Company makes no representations about the
 * suitability of this software for any purpose. It is provided
 * "as is" without express or implied warranty.
V2.1:0315 */
